/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/
package com.ibm.pim.webservices.samples;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.ibm.pim.services.common.Context;
import com.ibm.pim.services.common.ServiceResponse;
import com.ibm.pim.services.objects.Attribute;
import com.ibm.pim.services.objects.Category;
import com.ibm.pim.services.objects.Item;
import com.ibm.pim.services.objects.SearchPredicate;

/**
 * The class <code>SampleServiceTest</code> is ...
 * <p>
 *
 * @since 9.0.0
 *
 */
public class SampleServiceTest
{
    public static final String PIM_STRING_ENUM_ATTRIBUTE = "Stock Information/Order Quantities";
    public static final String PIM_NUM_ENUM_ATTRIBUTE = "Stock Information/Warehouse";
    public static String PIM_SPEC_NAME = "Product details";
    public static String PIM_SEC_SPEC_NAME = "Extra Range details";
    public static String PIM_LOOKUP_SPEC_NAME = "lk spec1";
    
    public static String PIM_CATALOG_NAME = "Products";
    public static String PIM_HIERARCHY_NAME = "Ranges";
    public static String PIM_HIERARCHY_NAME2 = "Specials";

    private static final String PIM_LOOKUP_TABLE_NAME = "colour lookup table";

    
    SampleService service = new SampleService();
    Context ctx = new Context("Admin", "trinitron", "samples");
    
    /**
     * @throws java.lang.Exception
     */
    @BeforeClass
    public static void setUpBeforeClass() throws Exception
    {
    }

    /**
     * @throws java.lang.Exception
     */
    @AfterClass
    public static void tearDownAfterClass() throws Exception
    {
    }

    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception
    {
    }

    /**
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception
    {
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#login(java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testLogin()
    {
        ServiceResponse response = service.login(ctx);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        assertEquals(2, response.getResults().size());
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getAllCatalogs(java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetAllCatalogs()
    {
        ServiceResponse response = service.getAllCatalogs(ctx);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        assertEquals(1, response.getResults().size());
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getAllCatalogsUsingSpec(java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetAllCatalogsUsingSpec()
    {
        ServiceResponse response = service.getAllCatalogsUsingSpec(ctx, PIM_SPEC_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        assertEquals(1, response.getResults().size());
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getAllLookupTablesUsingSpec(java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetAllLookupTablesUsingSpec()
    {
        ServiceResponse response = service.getAllLookupTablesUsingSpec(ctx, "lk spec1");
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getAllItemsInCatalog(java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetAllItemsInCatalog()
    {
        ServiceResponse response = service.getAllItemsInCatalog(ctx, PIM_CATALOG_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getAllEntriesInLookupTable(java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetAllEntriesInLookupTable()
    {
        ServiceResponse response = service.getAllEntriesInLookupTable(ctx, PIM_LOOKUP_TABLE_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getCategoriesForCatalog(java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetCategoriesForCatalog()
    {
        ServiceResponse response = service.getCategoriesForCatalog(ctx, PIM_CATALOG_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getLookupTableValues(java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetLookupTableValues()
    {
        ServiceResponse response = service.getLookupTableValues(ctx, PIM_LOOKUP_TABLE_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getTimezoneValues(java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetTimezoneValues()
    {
        ServiceResponse response = service.getTimezoneValues(ctx);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getNumberEnumValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetNumberEnumValues()
    {
        ServiceResponse response = service.getNumberEnumValues(ctx, PIM_SPEC_NAME, PIM_STRING_ENUM_ATTRIBUTE);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getStringEnumValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetStringEnumValues()
    {
        ServiceResponse response = service.getStringEnumValues(ctx, PIM_SPEC_NAME, PIM_NUM_ENUM_ATTRIBUTE);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getItemCategories(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetItemCategories()
    {
        ServiceResponse response = service.getItemCategories(ctx, "B 1", PIM_CATALOG_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getItemAttributeValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetItemAttributeValues()
    {
        ServiceResponse response = service.getItemAttributeValues(ctx, "B 1", PIM_CATALOG_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getLookupAttributeValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetLookupAttributeValues()
    {
        ServiceResponse response = service.getLookupAttributeValues(ctx, "Black", PIM_LOOKUP_TABLE_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getCategoryAttributeValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetCategoryAttributeValues()
    {
        ServiceResponse response = service.getCategoryAttributeValues(ctx, "Standard", PIM_HIERARCHY_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#getDynamicItemAttributeValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testGetDynamicItemAttributeValues()
    {
        ServiceResponse response = service.getDynamicItemAttributeValues(ctx, "B 1", PIM_CATALOG_NAME);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
        List<String> results = response.getResults();
        assertTrue(results.size() > 0);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#searchItemsInCatalog(java.lang.String, java.lang.String, java.lang.String, java.lang.String[], java.lang.String, java.lang.String[], java.lang.String[], java.lang.String[], java.lang.String[])}.
     */
    @Test
    public final void testSearchItemsInCatalog()
    {
    	List<SearchPredicate> predicates = new ArrayList<SearchPredicate>();
    	SearchPredicate predicate = new SearchPredicate("Product details/ID", "%STRING", "%B 1%", "like");
    	predicates.add(predicate);
    	List<String> mappedToCategories = new ArrayList<String>();
    	mappedToCategories.add("Ranges/Deluxe");
        ServiceResponse response = service.searchItemsInCatalog(ctx, PIM_CATALOG_NAME, mappedToCategories, "true", predicates);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#setItemAttributeValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String[], java.lang.String[], java.lang.String[], java.lang.String[])}.
     */
    @Test
    public final void testSetItemAttributeValues()
    {
    	Item item = new Item();
    	item.setPrimaryKey("B 1");
    	item.setContainerName(PIM_CATALOG_NAME);
    	List<Attribute> attributes = new ArrayList<Attribute>();
    	attributes.add(new Attribute("Product details/Name", "STRING", "<div align=\"center\"><u>Product B 1a</u><br /></div>"));
    	attributes.add(new Attribute("Product details/Colour", "STRING_ENUMERATION", null));
    	attributes.add(new Attribute("Product details/Price", "CURRENCY", "19.01"));
    	attributes.add(new Attribute("Product details/Available", "FLAG", "true"));
    	attributes.add(new Attribute("Product details/Image", "IMAGE", "ibm.jpg"));
    	attributes.add(new Attribute("Product details/Stock Information/Stock", "INTEGER", "3"));
		attributes.add(new Attribute("Product details/Stock Information/Warehouse", "STRING_ENUMERATION", null));
		attributes.add(new Attribute("Product details/Stock Information/Leadtime", "NUMBER", "5.00"));
		attributes.add(new Attribute("Product details/Stock Information/Order Quantities", "NUMBER_ENUMERATION", null));
		attributes.add(new Attribute("Product details/Based on Product", "RELATIONSHIP", null));
		attributes.add(new Attribute("Product details/Database Sequence", "SEQUENCE", "1"));
		attributes.add(new Attribute("Product details/Web Image", "THUMBNAIL_IMAGE", "zongzi.jpg"));
		attributes.add(new Attribute("Product details/Availability Date", "DATE", "1274405351000"));
		attributes.add(new Attribute("Product details/Availability Timezone", "TIMEZONE", null));
		attributes.add(new Attribute("Product details/Product Website", "URL", "http://www.mypoducts.com/productA1.html"));
		attributes.add(new Attribute("Extra Range details/Long Description", "STRING", null));
		item.setAttributes(attributes);
		List<String> mappedToCategories = new ArrayList<String>();
		ServiceResponse response = service.setItemAttributeValues(ctx, item, mappedToCategories);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#setLookupAttributeValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String[], java.lang.String[], java.lang.String[])}.
     */
    @Test
    public final void testSetLookupAttributeValues()
    {
    	Item item = new Item();
    	item.setPrimaryKey("Black");
    	item.setContainerName("colour lookup table");
    	List<Attribute> attributes = new ArrayList<Attribute>();
    	attributes.add(new Attribute("lk-field1", "STRING", "Black"));
    	attributes.add(new Attribute("lk-field2", "NUMBER", "2.00"));
    	item.setAttributes(attributes);
        ServiceResponse response = service.setLookupAttributeValues(ctx, item);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#setCategoryAttributeValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String[], java.lang.String[], java.lang.String[])}.
     */
    @Test
    public final void testSetCategoryAttributeValues()
    {
    	Category category = new Category();
    	category.setPrimaryKey("Deluxe");
    	category.setCategoryPath("Deluxe");
    	category.setContainerName(PIM_HIERARCHY_NAME);
    	List<Attribute> attributes = new ArrayList<Attribute>();
    	category.setAttributes(attributes);
        ServiceResponse response = service.setCategoryAttributeValues(ctx, category);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

	String itemCreatedPK = "pk" + new Date();
    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#createItem(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String[], java.lang.String[], java.lang.String[], java.lang.String[])}.
     */
    @Test
    public final void testCreateItem()
    {
    	Item item = new Item();
    	item.setPrimaryKey(itemCreatedPK);
    	item.setContainerName(PIM_CATALOG_NAME);
    	List<Attribute> attributes = new ArrayList<Attribute>();
    	attributes.add(new Attribute("Product details/ID", "STRING", itemCreatedPK));
    	attributes.add(new Attribute("Product details/Name", "STRING", "<div align=\"center\"><u>Product A 1x</u><br /></div>"));
    	attributes.add(new Attribute("Product details/Colour", "STRING_ENUMERATION", null));
    	attributes.add(new Attribute("Product details/Price", "CURRENCY", "19.01"));
    	attributes.add(new Attribute("Product details/Available", "FLAG", "true"));
    	attributes.add(new Attribute("Product details/Image", "IMAGE", "ibm.jpg"));
    	attributes.add(new Attribute("Product details/Stock Information/Stock", "INTEGER", "3"));
		attributes.add(new Attribute("Product details/Stock Information/Warehouse", "STRING_ENUMERATION", null));
		attributes.add(new Attribute("Product details/Stock Information/Leadtime", "NUMBER", "5.00"));
		attributes.add(new Attribute("Product details/Stock Information/Order Quantities", "NUMBER_ENUMERATION", null));
		attributes.add(new Attribute("Product details/Based on Product", "RELATIONSHIP", null));
		attributes.add(new Attribute("Product details/Database Sequence", "SEQUENCE", "1"));
		attributes.add(new Attribute("Product details/Web Image", "THUMBNAIL_IMAGE", "zongzi.jpg"));
		attributes.add(new Attribute("Product details/Availability Date", "DATE", "1274405351000"));
		attributes.add(new Attribute("Product details/Availability Timezone", "TIMEZONE", null));
		attributes.add(new Attribute("Product details/Product Website", "URL", "http://www.mypoducts.com/productA1.html"));
		attributes.add(new Attribute("Extra Range details/Long Description", "STRING", null));
		item.setAttributes(attributes);
		List<String> mappedToCategories = new ArrayList<String>();
        ServiceResponse response = service.createItem(ctx, item, mappedToCategories);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

	String lkpEntryCreatedKey = "Colour" + new Date();
    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#createLookupEntry(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String[], java.lang.String[], java.lang.String[])}.
     */
    @Test
    public final void testCreateLookupEntry()
    {
    	Item lkpEntry = new Item();
		lkpEntry.setPrimaryKey(lkpEntryCreatedKey);
    	lkpEntry.setContainerName("colour lookup table");
    	List<Attribute> attributes = new ArrayList<Attribute>();
    	attributes.add(new Attribute("lk-field1", "STRING", lkpEntryCreatedKey));
    	attributes.add(new Attribute("lk-field2", "NUMBER", "2.00"));
    	lkpEntry.setAttributes(attributes);
        ServiceResponse response = service.createLookupEntry(ctx, lkpEntry);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

	String categoryCreatedPath = "category" + new Date();
    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#createCategory(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String[], java.lang.String[], java.lang.String[])}.
     */
    @Test
    public final void testCreateCategory()
    {
    	Category category = new Category();
		category.setPrimaryKey(categoryCreatedPath);
		category.setCategoryPath(categoryCreatedPath);
    	category.setContainerName(PIM_HIERARCHY_NAME);
    	List<Attribute> attributes = new ArrayList<Attribute>();
    	category.setAttributes(attributes);
        ServiceResponse response = service.createCategory(ctx, category);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#deleteItem(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testDeleteItem()
    {
        Item item = new Item();
    	item.setPrimaryKey(itemCreatedPK);
    	item.setContainerName(PIM_CATALOG_NAME);
		ServiceResponse response = service.deleteItem(ctx, item);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#deleteLookupEntry(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testDeleteLookupEntry()
    {
    	Item lkpEntry = new Item();
    	lkpEntry.setPrimaryKey(lkpEntryCreatedKey);
    	lkpEntry.setContainerName("colour lookup table");
        ServiceResponse response = service.deleteLookupEntry(ctx, lkpEntry);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

    /**
     * Test method for {@link com.ibm.pim.webservices.samples.SampleService#deleteCategory(java.lang.String, java.lang.String, java.lang.String, java.lang.String)}.
     */
    @Test
    public final void testDeleteCategory()
    {
    	Category category = new Category();
    	category.setPrimaryKey(categoryCreatedPath);
		category.setCategoryPath(categoryCreatedPath);
    	category.setContainerName(PIM_HIERARCHY_NAME);
        ServiceResponse response = service.deleteCategory(ctx, category);
        assertTrue(response.getStatus() == ServiceResponse.SUCCEEDED);
    }

}
