/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.clients;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import javax.xml.rpc.ServiceException;

import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.i18n.Messages;
import org.apache.axis.message.SOAPBodyElement;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.soap.SOAPConstants;
import org.xml.sax.SAXException;

public class SoapClient {
    
	public SoapClient()
    {
        //NOOP
    }
    
    /**
     * @param args
     */
    public static void main(String[] args)
    {
        SoapClient client = new SoapClient();
        
        String endpointUrl = "http://192.168.153.129:9081/ProductWS_HTTPRouter/services/ProductPort";

    	String requestXmlStr = "<?xml version='1.0' encoding='UTF-8'?> "
    		+ "<soapenv:Envelope xmlns:soapenv='http://schemas.xmlsoap.org/soap/envelope/'> "
    		+ "<soapenv:Header> " 
    		+ "  <wsse:Security xmlns:wsse='http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd' soapenv:mustUnderstand='1'> " 
    		+ "     <wsse:UsernameToken> "  
    		+ "        <wsse:Username>cusadmin</wsse:Username> "
    		+ "        <wsse:Password>cusadmin</wsse:Password> "
    		+ "     </wsse:UsernameToken> "
    		+ "  </wsse:Security> "
    		+ "</soapenv:Header> "
    		+ "<soapenv:Body> "
    		+ "  <port:AddFinancialProduct xmlns:soapenc='http://schemas.xmlsoap.org/soap/encoding/' xmlns:common='http://www.ibm.com/xmlns/prod/websphere/wcc/common/schema' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xs='http://www.w3.org/2001/XMLSchema' xmlns:product='http://www.ibm.com/xmlns/prod/websphere/mdm/product/schema' xmlns:partybiz='http://www.ibm.com/xmlns/prod/websphere/wcc/partybiz/schema' xmlns:common-intf='http://www.ibm.com/xmlns/prod/websphere/wcc/common/intf/schema' xmlns:port='http://www.ibm.com/xmlns/prod/websphere/mdm/product/port'> "
    		+ "     <control> "
    		+ "        <requestId>12346</requestId> "
    		+ "        <requesterName>cusadmin</requesterName> "
    		+ "        <requesterLanguage>100</requesterLanguage> "
    		+ "     </control> "
    		+ "     <request> "
    		+ "        <productTypeId>4</productTypeId> "
    		+ "        <name>Sample Product Test</name> "
    		+ "        <shortDescription>SPT</shortDescription> "
    		+ "        <description>Sample Product Test</description> "
    		+ "        <productStructureType code='1'></productStructureType> "
    		+ "     </request> "
    		+ "  </port:AddFinancialProduct> " 
    		+ "</soapenv:Body> "
    		+ "</soapenv:Envelope> ";
        
    	boolean bodyInStream = false;
    	
        String soapActionURI = "";      //this parameter is required for SOAP 1.1, skipping assuming SOAP 1.2 based test    	
        String responseMsg = client.invokeService(endpointUrl, requestXmlStr, bodyInStream, soapActionURI);
        System.out.println("response message: \n" + responseMsg);
    }
    
    @SuppressWarnings("unchecked")
    public String invokeService(String url, String soapRequest, boolean bodyInStream, String soapActionURI)
    {
        Call call = null;
        
        //prepare service call
        try
        {
            Service service = new Service();
            call = (Call) service.createCall();
            call.setTargetEndpointAddress(new URL(url));     
            //setting of SOAPAction is mandatory for SOAP 1.1, whereas it is optional in SOAP 1.2
            //To provide support for SOAP 1.1, set if provided.
            if (soapActionURI != null && !soapActionURI.isEmpty()) {
                call.setSOAPActionURI(soapActionURI);
            }
        }
        catch (ServiceException se)
        {
        	return "\nUnable to create Call object due to : " + se.getClass();
        }
        catch (MalformedURLException mue)
        {
        	return "\nBad web service URL due to : " + mue.getClass();
        }

        // call service
        try {
            InputStream fis = new ByteArrayInputStream(soapRequest.getBytes());
            SOAPEnvelope requestEnvelope;
            if (bodyInStream)
            {
                SOAPConstants soapConsts = call.getMessageContext().getSOAPConstants();
                requestEnvelope = new SOAPEnvelope(soapConsts, call.getMessageContext().getSchemaVersion());

                SOAPBodyElement elmt = new SOAPBodyElement(fis);
                requestEnvelope.addBodyElement(elmt);
            }
            else
            {
                requestEnvelope = new SOAPEnvelope(fis);
            }

            Message requestMessage = new Message(requestEnvelope, bodyInStream);

            call.setRequestMessage(requestMessage);
			call.invoke();
        }
        catch (SAXException e)
        {
            e.printStackTrace();
        	return "\nUnable to create SOAP Body element due to exception: " + e.getClass();
		} 
        catch (AxisFault af) {
            af.printStackTrace();
        	return "\nUnable to invoke a service call due to exception: " + af.getClass();
		}

        // process response
        try
        {
        	Message responseMessage = call.getMessageContext().getResponseMessage();
            if (responseMessage == null)
            {
                if (call.getMessageContext().isPropertyTrue(Call.FAULT_ON_NO_RESPONSE, false))
                {
                    throw new AxisFault(Messages.getMessage("nullResponse00"));
                }
                else
                {
                    return null;
                }
            }

            SOAPEnvelope responseEnvelope = responseMessage.getSOAPEnvelope();
            Vector v = responseEnvelope.getBodyElements();
            StringBuffer buff = new StringBuffer();
            for (int i = 0; i < v.size(); i++)
            {
                buff.append(((SOAPBodyElement) v.get(i)).toString());
            }
            return buff.toString();
        } 
        catch (AxisFault e) {
            e.printStackTrace();
        	return "\nUnable to invoke a service Call due to exception: " + e.getClass();
		}
    }
}
