/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.common;

/**
 * The class <code>Context</code> is a common object 
 * for web service context settings.
 * <p>
 *
 * @author maobing
 * @since 9.0.0
 *
 */
public class Context
{
    private String user;
    private String password;
    private String company;

    /**
     * Default constructor.
     */
    public Context()
    {
        this.user = null;
        this.password = null;
        this.company = null;
    }

    /**
     * Construct a context object with user/password/company.
     * <p>
     * @param userAndCompany
     * @param passwd
     */
    public Context(String userAndCompany, String password)
    {
        String[] split = userAndCompany.split("@");
        if (split == null || split.length < 2)
        {
            throw new IllegalArgumentException("Incorrect userAndCompany.");
        }
        this.user = split[0].trim();
        this.password = password;
        this.company = split[1].trim();
    }

    /**
     * Construct a context object with user/password/company.
     * <p>
     * @param user
     * @param passwd
     * @param company
     */
    public Context(String user, String password, String company)
    {
        this.user = user;
        this.password = password;
        this.company = company;
    }

    /**
     * @return Returns the user.
     */
    public String getUser()
    {
        return user;
    }

    /**
     * @param user The user to set.
     */
    public void setUser(String user)
    {
        this.user = user;
    }

    /**
     * @return Returns the password.
     */
    public String getPassword()
    {
        return password;
    }

    /**
     * @param passwd The password to set.
     */
    public void setPassword(String password)
    {
        this.password = password;
    }

    /**
     * @return Returns the company.
     */
    public String getCompany()
    {
        return company;
    }

    /**
     * @param company The company to set.
     */
    public void setCompany(String company)
    {
        this.company = company;
    }

    private com.ibm.pim.context.Context pimContext = null;
    public com.ibm.pim.context.Context toPIMObject() throws Exception
    {
    	if (pimContext == null)
    	{
    		pimContext = com.ibm.pim.context.PIMContextFactory.getContext(this.getUser(), this.getPassword(), this.getCompany());
    	}
		return pimContext;
    }
}
