/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.impl;

import javax.jws.WebMethod;
import javax.jws.soap.SOAPBinding;
import com.ibm.pim.services.common.Context;
import com.ibm.pim.services.objects.Catalog;

/**
 * The class <code>CatalogService</code> is the implementation for Catalog services.
 * <p>
 *
 * @author maobing, rohit
 * @since 9.0.0
 *
 */
@javax.jws.WebService (
   targetNamespace="http://services.pim.ibm.com/",
   serviceName="CatalogService",
   portName="CatalogPort")
                                           
@SOAPBinding(
   style=SOAPBinding.Style.DOCUMENT,
   use=SOAPBinding.Use.LITERAL,
   parameterStyle=SOAPBinding.ParameterStyle.WRAPPED)
public class CatalogService
{
    /**
     * Default constructor.
     */
    public CatalogService()
    {
    }

    @WebMethod
    public Catalog createCatalog( Context ctx, Catalog catalog)
    {
        if(ctx == null || catalog == null){
            return null;
        }else if (catalog.getName() == null || catalog.getPrimarySpec().getName() == null || 
                catalog.getPrimaryHierarchy().getName() == null ){
            return null;
        }
        
        com.ibm.pim.context.Context context = null;
        Catalog retCatalog = null;
        
        try 
        {
            // PIM context
            context = ctx.toPIMObject();
            
            // get PIM Managers.
            com.ibm.pim.catalog.CatalogManager catalogManager = context.getCatalogManager();
            com.ibm.pim.spec.SpecManager specManager = context.getSpecManager();
            com.ibm.pim.hierarchy.HierarchyManager hieararchyManager = context.getHierarchyManager();
            com.ibm.pim.organization.OrganizationManager organizationManager = context.getOrganizationManager(); 
            
            // create catalog parameters
            String catalogName = catalog.getName();
            com.ibm.pim.spec.PrimarySpec primarySpec = (com.ibm.pim.spec.PrimarySpec)specManager.getSpec( catalog.getPrimarySpec().getName() );
            com.ibm.pim.hierarchy.Hierarchy primaryHierarcy = hieararchyManager.getHierarchy( catalog.getPrimaryHierarchy().getName() );
            com.ibm.pim.organization.AccessControlGroup acg = null;
            
            com.ibm.pim.catalog.Catalog pimCatalog = null; // PIM Catalog.
            if( catalog.getAccessControlGroup() != null && catalog.getAccessControlGroup().getName()!=null ){
                acg = organizationManager.getAccessControlGroup( catalog.getAccessControlGroup().getName() );
                if(acg!=null){
                    pimCatalog = catalogManager.createCatalog(primarySpec, catalogName, primaryHierarcy, acg );
                }
            }else{ //acg is null
                pimCatalog = catalogManager.createCatalog(primarySpec, catalogName, primaryHierarcy);
            }
            
            // display attribute - belongs to the primary Spec
            if(catalog.getDisplayAttributeName()!=null){
                com.ibm.pim.attribute.AttributeDefinition displayAttribute = primarySpec.getAttributeDefinition(catalog.getDisplayAttributeName());
                if(displayAttribute != null){
                    pimCatalog.setDisplayAttribute(displayAttribute);
                }else{
                    return null;
                }
            }
            
            if( pimCatalog.save() ){            
                retCatalog = new Catalog(pimCatalog);
            }
            
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                context.cleanUp();
        }
        return retCatalog;
    }

    @WebMethod
    public Catalog updateCatalog(Context ctx, Catalog catalog)
    {
        if(ctx == null || catalog == null){
            return null;
        }else if (catalog.getName() == null ){
            return null;
        }
        
        com.ibm.pim.context.Context context = null;
        Catalog retCatalog = null;
        
        try 
        {
            // PIM Context.
            context = ctx.toPIMObject();
            
            // PIM managers
            com.ibm.pim.catalog.CatalogManager catalogManager = context.getCatalogManager();
            com.ibm.pim.organization.OrganizationManager organizationManager = context.getOrganizationManager(); 

            // PIM Catalog.
            com.ibm.pim.catalog.Catalog pimCatalog = catalogManager.getCatalog( catalog.getName() );
            
            // Update Catalog:
            
            // update acg
            if(catalog.getAccessControlGroup()!=null){
                pimCatalog.setAccessControlGroup( organizationManager.getAccessControlGroup(
                                                                          catalog.getAccessControlGroup().getName())  );
            }
            
            //update display attribute - belongs to the primary Spec
            if(catalog.getDisplayAttributeName()!=null){
                com.ibm.pim.attribute.AttributeDefinition displayAttribute = 
                        pimCatalog.getSpec().getAttributeDefinition( catalog.getDisplayAttributeName());
                if(displayAttribute != null){
                    pimCatalog.setDisplayAttribute(displayAttribute);
                }
            }

            //more attributes
            setCatalogAttributes(context, catalog, pimCatalog);
            
            //save
            if( pimCatalog.save() ){            
                retCatalog = new Catalog(pimCatalog);
            }
            
        }
        catch (Exception e){
            e.printStackTrace();
        }
        finally{
            if(ctx!=null)
                context.cleanUp();
        }
        
        return retCatalog;
    }

    @WebMethod
    public Catalog getCatalog(Context ctx, Catalog catalog)
    {
        
        if(ctx == null || catalog == null){
            return null;
        }else if (catalog.getName() == null ){
            return null;
        }
        
        com.ibm.pim.context.Context context = null;
        Catalog retCatalog = null; 

        try 
        {
            // PIM context
            context = ctx.toPIMObject();
            
            // PIM CatalogManager.
            com.ibm.pim.catalog.CatalogManager catalogManager = context.getCatalogManager();
            
            // PIM Catalog.
            com.ibm.pim.catalog.Catalog pimCatalog = catalogManager.getCatalog( catalog.getName() );
            
            // to service Item. 
            retCatalog = new Catalog(pimCatalog);     
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(context!=null)
                context.cleanUp();
        }
        
        return retCatalog;
    }

    @WebMethod
    public boolean deleteCatalog(Context ctx, Catalog catalog)
    {
        
        /*if(ctx == null || catalog == null){
            return false;
        }else if (catalog.getName() == null ){
            return false;
        }
        
        com.ibm.pim.context.Context context = null;
        boolean bRet = false;
        try 
        {
            // PIM context
            context = ctx.toPIMObject();
            
            // PIM CatalogManager.
            // com.ibm.pim.catalog.CatalogManager catalogManager = context.getCatalogManager();
            
            // PIM Catalog.
            // com.ibm.pim.catalog.Catalog pimCatalog = catalogManager.getCatalog(catalog.getName());
            
            // Delete Item.
            // bRet = pimCatalog.deleteAsynchronous() != null;               
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(context!=null)
                context.cleanUp();
        }
        return bRet;*/
        return false;
    }
    
    private void setCatalogAttributes(com.ibm.pim.context.Context ctx, Catalog catalog, com.ibm.pim.catalog.Catalog pimCatalog){
        
        if( pimCatalog == null ){ //ctx and catalog are null checked before this method
            return;
        }
        
        // create catalog parameters
        com.ibm.pim.hierarchy.HierarchyManager hieararchyManager = ctx.getHierarchyManager();

        //add secondarys hierarchies
        if(catalog.getAddSecondaryHierarchies()!=null){         
            for (int i = 0; i < catalog.getAddSecondaryHierarchies().length; i++) {
                if(catalog.getAddSecondaryHierarchies()[i]!=null){
                    com.ibm.pim.hierarchy.Hierarchy pimHierarchy = hieararchyManager.getHierarchy(catalog.getAddSecondaryHierarchies()[i].getName());
                    if(pimHierarchy!=null){
                        pimCatalog.addSecondaryHierarchy( pimHierarchy );
                    }
                }
            }           
        }
        
        //remove secondarys hierarchies
        if(catalog.getRemoveSecondaryHierarchies()!=null){
            for (int i = 0; i < catalog.getRemoveSecondaryHierarchies().length; i++) {
                if(catalog.getRemoveSecondaryHierarchies()[i]!=null){
                    com.ibm.pim.hierarchy.Hierarchy pimHierarchy = hieararchyManager.getHierarchy(catalog.getRemoveSecondaryHierarchies()[i].getName());
                    if(pimHierarchy!=null){
                        pimCatalog.removeSecondaryHierarchy( pimHierarchy );
                    }
                }
            }
        }
        
        //custom parameters add
        String[] customParamKeys = catalog.getCustomParameterKeys();
        String[] customParamValues = catalog.getCustomParameterValues();
        if(customParamKeys!=null && customParamValues!=null && customParamKeys.length == customParamValues.length){
            for(int i=0; i< customParamKeys.length; i++){
                pimCatalog.addCustomParameterValue(customParamKeys[i], customParamValues[i]);
            }
        }
        
        //custom parameters remove
        String[] deleteCustomParamKeys = catalog.getCustomParameterKeys();
        if(deleteCustomParamKeys!=null){
            for(int i=0; i< deleteCustomParamKeys.length; i++){
                pimCatalog.deleteCustomParameter(deleteCustomParamKeys[i]);
            }
        }
        
        //version name
        if(catalog.getVersionName()!=null){
            pimCatalog.createVersion( catalog.getVersionName() );
        }
        
        //isCached
        if(catalog.isCached()!=null){
            pimCatalog.setCached( catalog.isCached() );
        }
        
    }
    
}