/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.impl;

import javax.jws.WebMethod;
import javax.jws.soap.SOAPBinding;

import com.ibm.pim.services.common.Context;
import com.ibm.pim.services.objects.Hierarchy;

/**
 * The class <code>HierarchyService</code> is ...
 * <p>
 *
 * @since 9.0.0
 *
 */
@javax.jws.WebService (
   targetNamespace="http://services.pim.ibm.com/",
   serviceName="HierarchyService",
   portName="HierarchyPort")
                                           
@SOAPBinding(
   style=SOAPBinding.Style.DOCUMENT,
   use=SOAPBinding.Use.LITERAL,
   parameterStyle=SOAPBinding.ParameterStyle.WRAPPED)
public class HierarchyService
{
    /**
     * Default constructor.
     */
    public HierarchyService()
    {
    }

    @WebMethod
    public Hierarchy createHierarchy(Context ctx, Hierarchy hierarchy)
    {
        com.ibm.pim.context.Context context = null;
        Hierarchy returnHierarchy = null;
        try
        {
            //setting the context
            context = ctx.toPIMObject();
            
            // PIM HierarchyManager, Organization Manager, DocStore Manager, Spec Manager, AttributeCollection Manager.
            com.ibm.pim.hierarchy.HierarchyManager hierarchyManager = context.getHierarchyManager();
            com.ibm.pim.organization.OrganizationManager organizationManager = context.getOrganizationManager();
            
            //create supporting objects
            com.ibm.pim.spec.SpecManager specManager = context.getSpecManager();
            com.ibm.pim.spec.PrimarySpec primarySpec = (com.ibm.pim.spec.PrimarySpec)specManager.getSpec(hierarchy.getSpec().getName());;
            
            com.ibm.pim.attribute.AttributeDefinition pathAttribute = null;
            if(hierarchy.getPathAttributeName()!=null)
                pathAttribute = primarySpec.getAttributeDefinition(hierarchy.getPathAttributeName());
            
            com.ibm.pim.attribute.AttributeDefinition displayAttribute = null;
            if(hierarchy.getDisplayAttributeName()!=null)
                displayAttribute = primarySpec.getAttributeDefinition(hierarchy.getDisplayAttributeName());
            
            com.ibm.pim.organization.AccessControlGroup acg = null;
            if(hierarchy.getAccessControlGroup()!=null)
                acg = organizationManager.getAccessControlGroup(hierarchy.getAccessControlGroup().getName());
            
            //creating PIM hierarchy object
            com.ibm.pim.hierarchy.Hierarchy pimHierarchy = null;
            
            if(hierarchy.getDisplayAttributeName()!=null && hierarchy.getPathAttributeName()!=null && acg!=null)
                pimHierarchy = hierarchyManager.createHierarchy(primarySpec, hierarchy.getName(), pathAttribute, acg, displayAttribute);
            else if(hierarchy.getPathAttributeName()!=null)
                pimHierarchy = hierarchyManager.createHierarchy(primarySpec, hierarchy.getName(), pathAttribute);  
            else
                pimHierarchy = hierarchyManager.createHierarchy(primarySpec, hierarchy.getName());
            
            //setting parameters
            if(acg!=null)
                pimHierarchy.setAccessControlGroup(acg);
            if(displayAttribute!=null)
                pimHierarchy.setDisplayAttribute(displayAttribute);
            
            //saving the new created hierarchy
            pimHierarchy.save();
            
            returnHierarchy = new Hierarchy(pimHierarchy);
            
        }

        
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                context.cleanUp();
        }
        return returnHierarchy;
    }

    @WebMethod
    public Hierarchy updateHierarchy(Context ctx, Hierarchy hierarchy)
    {
        com.ibm.pim.context.Context context = null;
        Hierarchy returnHierarchy = null;
        try
        {
          //setting the context
            context = ctx.toPIMObject();
            
            // PIM HierarchyManager, Organization Manager, DocStore Manager, Spec Manager, AttributeCollection Manager.
            com.ibm.pim.hierarchy.HierarchyManager hierarchyManager = context.getHierarchyManager();
            com.ibm.pim.organization.OrganizationManager organizationManager = context.getOrganizationManager();
            com.ibm.pim.spec.SpecManager specManager = context.getSpecManager();
            
            //creating pimHierarchy object
            com.ibm.pim.hierarchy.Hierarchy pimHierarchy = hierarchyManager.getHierarchy(hierarchy.getName());
            com.ibm.pim.spec.PrimarySpec primarySpec = (com.ibm.pim.spec.PrimarySpec)specManager.getSpec(hierarchy.getSpec().getName());
            com.ibm.pim.attribute.AttributeDefinition displayAttribute = null;
            
            com.ibm.pim.organization.AccessControlGroup acg = null;
            if(hierarchy.getAccessControlGroup()!=null)
                acg = organizationManager.getAccessControlGroup(hierarchy.getAccessControlGroup().getName());
            
            if(hierarchy.getDisplayAttributeName()!=null)
                displayAttribute = primarySpec.getAttributeDefinition(hierarchy.getDisplayAttributeName());
            
            //setting parameters
            if(acg!=null)
                pimHierarchy.setAccessControlGroup(acg);
            
            if(displayAttribute!=null)
                pimHierarchy.setDisplayAttribute(displayAttribute);
            
            //saving the new created hierarchy
            pimHierarchy.save();
            
            returnHierarchy = new Hierarchy(pimHierarchy);
            
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                context.cleanUp();
        }
        return returnHierarchy;
    }

    @WebMethod
    public Hierarchy getHierarchy(Context ctx, Hierarchy hierarchy)
    {
        com.ibm.pim.context.Context context = null;
        Hierarchy returnHierarchy = null;
        try{
            //setting context
            context = ctx.toPIMObject();
            
            // PIM HierarchyManager, Organization Manager, DocStore Manager, Spec Manager, AttributeCollection Manager.
            com.ibm.pim.hierarchy.HierarchyManager hierarchyManager = context.getHierarchyManager();
            
            // get PIM Hierarchy.
            
            if(hierarchy.getName()!=null)
            {
                com.ibm.pim.hierarchy.Hierarchy pimHierarchy = hierarchyManager.getHierarchy(hierarchy.getName());
                returnHierarchy = new Hierarchy(pimHierarchy);
            }
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                context.cleanUp();
        }
        return returnHierarchy;
    }

    @WebMethod
    public void deleteHierarchy(Context ctx, Hierarchy hierarchy)
    {
        com.ibm.pim.context.Context context = null;
        try{
            //setting context
            context = ctx.toPIMObject();
            
            // PIM HierarchyManager, Organization Manager, DocStore Manager, Spec Manager, AttributeCollection Manager.
            com.ibm.pim.hierarchy.HierarchyManager hierarchyManager = context.getHierarchyManager();
            
            // get PIM Hierarchy.
            com.ibm.pim.hierarchy.Hierarchy pimHierarchy = hierarchyManager.getHierarchy(hierarchy.getName());            
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                context.cleanUp();
        }
    }
}
