/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.impl;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.soap.SOAPBinding;

import com.ibm.pim.services.common.Context;
import com.ibm.pim.services.objects.Attribute;
import com.ibm.pim.services.objects.Item;

/**
 * The class <code>ProductService</code> is for the product web service.
 * <p>
 *
 * @author maobing, uttama
 * @since 9.0.0
 *
 */
@javax.jws.WebService (
   targetNamespace="http://services.pim.ibm.com/",
   serviceName="ProductService",
   portName="ProductPort")
                       
@SOAPBinding(
   style=SOAPBinding.Style.DOCUMENT,
   use=SOAPBinding.Use.LITERAL,
   parameterStyle=SOAPBinding.ParameterStyle.WRAPPED)
public class ProductService
{

    /**
     * Default constructor.
     */
    public ProductService()
    {
    }

    /**
     * Create an item with the given data.
     * <p>
     * @param item the given item data.
     * @return the new item.
     * @param wsContext the given context.
     */
    @WebMethod
    public Item createItem(Item item, Context wsContext)
    {
    	com.ibm.pim.context.Context ctx = null;
        try 
        {
            // PIM context
        	ctx = wsContext.toPIMObject();
                        
            // PIM CatalogManager.
            com.ibm.pim.catalog.CatalogManager catalogManager = ctx.getCatalogManager();
            
            // PIM Catalog.
            com.ibm.pim.catalog.Catalog catalog = catalogManager.getCatalog(item.getContainerName());
            
            // create PIM Item. 
            com.ibm.pim.catalog.item.Item pimItem =  catalog.createItem();            
            pimItem.setPrimaryKey(item.getPrimaryKey());            
            pimItem.setAttributeValue(catalog.getDisplayAttribute().getPath(), item.getDisplayName());            
            setAttributes(item, pimItem); 
            pimItem.save();
            
            // set Id to service Item.
            item.setId(pimItem.getId());
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                ctx.cleanUp();
        }
        return item;
    }   
    

    /**
     * Update the given item with the given data.
     * <p>
     * @param item the given item and its new data.
     * @return the updated item and its data.
     * @param wsContext the given context.
     */
    @WebMethod
    public Item updateItem(Item item, Context wsContext)
    {
    	com.ibm.pim.context.Context ctx = null;
        try 
        {
            // PIM Context.
        	ctx = wsContext.toPIMObject();
            
            // PIM CatalogManager.
            com.ibm.pim.catalog.CatalogManager catalogManager = ctx.getCatalogManager();
            
            // PIM Catalog. 
            com.ibm.pim.catalog.Catalog catalog = catalogManager.getCatalog(item.getContainerName());
            
            // PIM Item.
            com.ibm.pim.catalog.item.Item pimItem = catalog.getItemByPrimaryKey(item.getPrimaryKey());
            
            // Update Item.                   
            pimItem.setAttributeValue(catalog.getDisplayAttribute().getPath(), item.getDisplayName());
            setAttributes(item, pimItem);                    
            pimItem.save();   
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                ctx.cleanUp();
        }
        return item;
    }

    /**
     * Retrieve the give item.
     * <p>
     * @param item the given item.
     * @return the given item and its data.
     * @param wsContext the given context.
     */
    @WebMethod
    public Item getItem(Item item, Context wsContext)
    {
    	com.ibm.pim.context.Context ctx = null;
        Item newItem = null; 
        try 
        {
            // PIM context
        	ctx = wsContext.toPIMObject();
            
            // PIM CatalogManager.
            com.ibm.pim.catalog.CatalogManager catalogManager = ctx.getCatalogManager();
            
            // PIM Catalog.
            com.ibm.pim.catalog.Catalog catalog = catalogManager.getCatalog(item.getContainerName());
            
            // Get PIM Item.
            com.ibm.pim.catalog.item.Item pimItem = catalog.getItemByPrimaryKey(item.getPrimaryKey());
            
            // to service Item. 
            newItem = new Item(pimItem);            
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                ctx.cleanUp();
        }
        return newItem;
    }

    /**
     * Delete the given item.
     * <p>
     * @param item the given item.
     * @param wsContext the given context.
     */
    @WebMethod
    public void deleteItem(Item item, Context wsContext)
    {
    	com.ibm.pim.context.Context ctx = null;
        try 
        {
            // PIM context
        	ctx = wsContext.toPIMObject();
            
            // PIM CatalogManager.
            com.ibm.pim.catalog.CatalogManager catalogManager = ctx.getCatalogManager();
            
            // PIM Catalog.
            com.ibm.pim.catalog.Catalog catalog = catalogManager.getCatalog(item.getContainerName());
            
            // Delete Item.
            com.ibm.pim.catalog.item.Item pimItem = catalog.getItemByPrimaryKey(item.getPrimaryKey());            
            pimItem.delete();               
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(ctx!=null)
                ctx.cleanUp();
        }
        return;
    }
    
    private void setAttributes(Item item, com.ibm.pim.catalog.item.Item pimItem)
    {
        List<Attribute> attributes = item.getAttributes();  
        for(Attribute attribute:attributes)
        {
            pimItem.setAttributeValue(attribute.getName(), attribute.getValue());
        }  
    }
}
