/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.impl;

import java.util.ArrayList;
import java.util.Collection;

import javax.jws.WebMethod;
import javax.jws.soap.SOAPBinding;

import com.ibm.pim.services.common.Context;
import com.ibm.pim.services.objects.Job;
import com.ibm.pim.services.objects.Report;
import com.ibm.pim.services.objects.Schedule;

/**
 * The class <code>Scheduler</code> is a sample web service for Scheduler.
 * <p>
 *
 * @author maobing, uttama
 * @since 9.0.0
 *
 */
@javax.jws.WebService (
    targetNamespace="http://samples.webservices.pim.ibm.com/",
    serviceName="SchedulerService",
    portName="SchedulerPort")
    
@SOAPBinding(
    style=SOAPBinding.Style.DOCUMENT,
    use=SOAPBinding.Use.LITERAL,
    parameterStyle=SOAPBinding.ParameterStyle.WRAPPED)
public class SchedulerService
{
    public SchedulerService()
    {
    }
    
    @WebMethod
    public Report createReport(Report report, Context wsContext)
    {
        Report newReport = null;
        
        try
        {
            // PIM context
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
    
            // report script
            com.ibm.pim.docstore.DocstoreManager docMgr = ctx.getDocstoreManager();
            com.ibm.pim.docstore.Document reportScript = docMgr.getDocument(report.getReportScriptFileName());
    
            // report spec
            com.ibm.pim.spec.SpecManager specMgr = ctx.getSpecManager();
            com.ibm.pim.spec.ScriptInputSpec reportInputSpec = (com.ibm.pim.spec.ScriptInputSpec)specMgr.getSpec(report.getInputSpecName());
            
            // report distribution
            com.ibm.pim.utils.RoutingManager rounteringMgr = ctx.getRoutingManager();
            com.ibm.pim.utils.Distribution distribution = rounteringMgr.getDistribution(report.getDistributionName());
            
            // get job manager. if failed, return null.
            com.ibm.pim.job.JobManager jobManager = ctx.getJobManager();
            com.ibm.pim.job.Report pimReport = jobManager.createReport(report.getName(), reportScript, reportInputSpec, distribution);
            
            pimReport.save();
            newReport = new Report(pimReport);
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        return newReport;
    }
    
    @WebMethod
    public Schedule createSchedule(Schedule schedule, Context wsContext)
    {
        Schedule newSchedule = null;
        
        try 
        {
            // PIM context
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
    
            // PIM job manager
            com.ibm.pim.job.JobManager jobManager = ctx.getJobManager();

            // PIM job
            com.ibm.pim.job.Job pimJob = getPIMJob(schedule.getJob(), ctx);
    
            // PIM schedule type
            com.ibm.pim.job.Schedule.Type type = getPIMScheduleType(schedule.getType());
            
            // PIM schedule
            com.ibm.pim.job.Schedule pimSchedule = null;
            if (type == null || type == com.ibm.pim.job.Schedule.Type.IMMEDIATE)
            {
                pimSchedule = jobManager.createSchedule(pimJob);
            }
            else if (type == null || type == com.ibm.pim.job.Schedule.Type.MINUTE)
            {
            	pimSchedule = jobManager.createSchedule(pimJob, schedule.getName(), schedule.getIntervalMinutes());
            }
            else
            {
                pimSchedule = jobManager.createSchedule(pimJob, schedule.getName(), type, schedule.getStartTime());
                
            }
           
            
            pimSchedule.save();
            // to service schedule
            newSchedule = new Schedule(pimSchedule);
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        return newSchedule;
    }

    @WebMethod
    public String getScheduleStatus(Schedule schedule, Context wsContext)
    {
        String status = null;
        int percentComplete ;
        try 
        {
            // PIM context
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
    
            // PIM schedule
            com.ibm.pim.job.Schedule pimSchedule = getPIMSchedule(schedule, ctx);
            
            // PIM schedule status
            com.ibm.pim.job.ScheduleStatus pimScheduleStatus = pimSchedule.getLatestCompletedStatus();  
            
            // Get percentage completion of currently running job
            if(pimScheduleStatus == null)
            {
                percentComplete = pimSchedule.getLatestRunCompletionPercentage();
                if(percentComplete > -1)
                {
                    status = percentComplete + "% Completed.";
                }                  
            }
            else
            {            
                // to service status
                status = pimScheduleStatus.getType().toString();
            }
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        return status;
    }
    
    @WebMethod
    public void stopSchedule(Schedule schedule, Context wsContext)
    {
        try 
        {
            // PIM context
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
    
            // PIM schedule
            com.ibm.pim.job.Schedule pimSchedule = getPIMSchedule(schedule, ctx);
            
            // stop schedule
            pimSchedule.stop();
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
    }
    
    @WebMethod
    public Job getJob(Job job, Context wsContext)
    {
        Job newJob = null;
        
        try 
        {
            // PIM context
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
    
            // PIM job
            com.ibm.pim.job.Job pimJob = getPIMJob(job, ctx);            
            
            // to service job.            
            newJob = new Job(pimJob);            
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        return newJob;
    }
    
    @WebMethod
    public Collection<Schedule> getAllSchedules(Job job, Context wsContext)
    {
        Collection<Schedule> schedules = new ArrayList<Schedule>();
        
        try 
        {
            // PIM context
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
    
            // PIM job
            com.ibm.pim.job.Job pimJob = getPIMJob(job, ctx);

            // PIM schedules
            Collection<com.ibm.pim.job.Schedule> pimSchedules = pimJob.getSchedules();
            
            // to service schedules
            for (com.ibm.pim.job.Schedule pimSchedule:pimSchedules)
            {
                schedules.add(new Schedule(pimSchedule));
            }
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        return schedules;
    }
    
    @WebMethod
    public Collection<Job> getAllJobs(Context wsContext)
    {
        Collection<Job> jobs = new ArrayList<Job>(); 
        
        try 
        {
            // PIM context
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
    
            // PIM JobManager.
            com.ibm.pim.job.JobManager jobManager = ctx.getJobManager();
        
            // PIM jobs
            Collection<com.ibm.pim.job.Job> pimJobs = jobManager.getJobs();
            
            // to service jobs
            for (com.ibm.pim.job.Job pimJob:pimJobs)
            {
                jobs.add(new Job(pimJob));
            }
        }
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        return jobs; 
    }

    private com.ibm.pim.job.Job getPIMJob(Job job, com.ibm.pim.context.Context ctx)
    {
        // PIM job manager
        com.ibm.pim.job.JobManager jobManager = ctx.getJobManager();

        // PIM job
        com.ibm.pim.job.Job pimJob = jobManager.getJob(job.getName());
        return pimJob;
    }

    private com.ibm.pim.job.Schedule getPIMSchedule(Schedule schedule, com.ibm.pim.context.Context ctx)
    {
        // PIM job
        com.ibm.pim.job.Job pimJob = getPIMJob(schedule.getJob(), ctx);

        // PIM schedules
        Collection<com.ibm.pim.job.Schedule> pimSchedules = pimJob.getSchedules();
        
        // PIM schedule
        com.ibm.pim.job.Schedule pimSchedule = null;
        for (com.ibm.pim.job.Schedule pimSchedule2:pimSchedules)
        {
            if (schedule.equals(pimSchedule2))
            {
                pimSchedule = pimSchedule2;
                break;
            }
        }
        //pimSchedule = pimJob.getLastRunSchedule();
        return pimSchedule;
    }
    
    private com.ibm.pim.job.Schedule.Type getPIMScheduleType(int type)
    {
        com.ibm.pim.job.Schedule.Type pimType = null;
        switch (type)
        {
            case Schedule.UNKNOWN_TYPE:
                return null;
            case Schedule.IMMEDIATE:
                return com.ibm.pim.job.Schedule.Type.IMMEDIATE;
            case Schedule.ONE_TIME:
                return com.ibm.pim.job.Schedule.Type.ONE_TIME;
            case Schedule.MINUTE:
                return com.ibm.pim.job.Schedule.Type.MINUTE;
            case Schedule.HOURLY:
                return com.ibm.pim.job.Schedule.Type.HOURLY;
            case Schedule.DAILY:
                return com.ibm.pim.job.Schedule.Type.DAILY;
            case Schedule.WEEKLY:
                return com.ibm.pim.job.Schedule.Type.WEEKLY;
            case Schedule.MONTHLY:
                return com.ibm.pim.job.Schedule.Type.MONTHLY;
            case Schedule.YEARLY:
                return com.ibm.pim.job.Schedule.Type.YEARLY;
        }
        
        return pimType;
    }
}
