/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.impl;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.soap.SOAPBinding;

import com.ibm.pim.services.common.Context;

/**
 * The class <code>SearchService</code> is for the search web service.
 * <p>
 *
 * @author maobing
 * @since 9.0.0
 *
 */
@javax.jws.WebService (
   targetNamespace="http://services.pim.ibm.com/",
   serviceName="SearchService",
   portName="SearchPort")
                       
@SOAPBinding(
   style=SOAPBinding.Style.DOCUMENT,
   use=SOAPBinding.Use.LITERAL,
   parameterStyle=SOAPBinding.ParameterStyle.WRAPPED)
public class SearchService
{

    /**
     * Default constructor.
     */
    public SearchService()
    {
    }

    /**
     * Search the system.
     * <p>
     * @param wql the query.
     * @return the result.
     * @param wsContext the given context.
     */
    @WebMethod
    public String search(String wql, Context wsContext)
    {
        try
        {
            com.ibm.pim.context.Context ctx = wsContext.toPIMObject();
            com.ibm.pim.search.SearchQuery wqlQuery = ctx.createSearchQuery(wql);
            com.ibm.pim.search.SearchResultSet rs = wqlQuery.execute();
            
            StringBuffer buf = new StringBuffer();
            buf.append("<results>\n");
            while (rs.next())
            {
                toXML(rs, wqlQuery.getColumnTypes(), buf);
            }
            buf.append("</results>\n");
            
            return buf.toString();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return null;
    }

    /*
     * Convert search results to XML string.
     */
    private void toXML(com.ibm.pim.search.SearchResultSet rs, List<com.ibm.pim.search.SearchQuery.ColumnType> columnTypes, StringBuffer xml) throws com.ibm.pim.common.exceptions.PIMSearchException
    {
        xml.append("\t<record>\n");
        int columnIndex = 1;
        for (com.ibm.pim.search.SearchQuery.ColumnType columnType:columnTypes)
        {
            toXMLColumn(rs, columnIndex++, columnType, xml);
        }
        xml.append("\t</record>\n");
    }

    private void toXMLColumn(com.ibm.pim.search.SearchResultSet rs, int columnIndex, com.ibm.pim.search.SearchQuery.ColumnType columnType, StringBuffer xml) throws com.ibm.pim.common.exceptions.PIMSearchException
    {
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.BOOL)
        {
            appendColumn(xml, "bool", "" + rs.getBoolean(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.CATALOG)
        {
            appendColumn(xml, "catalog", "" + rs.getInt(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.CATEGORY)
        {
            appendColumn(xml, "Category", "" + rs.getInt(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.DATE)
        {
            appendColumn(xml, "Date", "" + rs.getDate(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.HIERARCHY)
        {
            appendColumn(xml, "Hierarchy", "" + rs.getInt(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.ITEM)
        {
            appendColumn(xml, "Item", "" + rs.getInt(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.LOGENTRY)
        {
            appendColumn(xml, "LogEntry", "" + rs.getInt(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.NUMBER)
        {
            appendColumn(xml, "Number", "" + rs.getDouble(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.PK)
        {
            appendColumn(xml, "PK", "" + rs.getString(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.SPEC)
        {
            appendColumn(xml, "Spec", "" + rs.getInt(columnIndex));
            return;
        }
        if (columnType == com.ibm.pim.search.SearchQuery.ColumnType.STRING)
        {
            appendColumn(xml, "String", "" + rs.getString(columnIndex));
            return;
        }
    }
    
    private void appendColumn(StringBuffer xml, String type, String value)
    {
        xml.append("\t\t<column type=\"" + type + "\">");
        xml.append(value);
        xml.append("</column>\n");
    }
}
