/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.impl;

import javax.jws.WebMethod;
import javax.jws.soap.SOAPBinding;
import com.ibm.pim.services.common.Context;
import com.ibm.pim.services.objects.Spec;
import com.ibm.pim.services.utils.SpecHelper;
import com.ibm.pim.spec.LookupSpec;
import com.ibm.pim.spec.PrimarySpec;

/**
 * The class <code>SpecService</code> is ...
 * <p>
 *
 * @since 9.0.0
 *
 */
@javax.jws.WebService (
   targetNamespace="http://services.pim.ibm.com/",
   serviceName="SpecService",
   portName="SpecPort")
                                           
@SOAPBinding(
   style=SOAPBinding.Style.DOCUMENT,
   use=SOAPBinding.Use.LITERAL,
   parameterStyle=SOAPBinding.ParameterStyle.WRAPPED)
public class SpecService
{
    /**
     * Default constructor.
     */
    public SpecService()
    {
    }

    @WebMethod
    public Spec createSpec(Context ctx, Spec spec)
    {    	
        if(ctx == null || spec == null){
            return null;
        }else if (spec.getName() == null || spec.getType() == null || spec.getAttributes().size()==0){
            return null;
        }
        if(spec.getAttributes().get(0).getName()==null){
            return null; //primary key has to be specified
        }
    	com.ibm.pim.context.Context context = null;
    	Spec retSpec = null;
    	try{
    		
    		context = ctx.toPIMObject();

    		// PIM context
    		com.ibm.pim.spec.SpecManager specManager = context.getSpecManager();
    		
    		// create PIM Spec.
    		SpecHelper specHelper = new SpecHelper();
    		
    		//create spec
    		com.ibm.pim.spec.Spec pimSpec = null;
    		if(spec.getType().equals("FILE_SPEC")){
    		    if(spec.getFileSpecType()==null){
    		        return null;
    		    }
    			pimSpec = specManager.createFileSpec(spec.getName(), specHelper.stringToFileType(spec.getFileSpecType()));
			}else{
				pimSpec = specManager.createSpec( spec.getName(), specHelper.stringToSpecType(spec.getType()) );
			}
            
            //set 
    		if(spec.isLocalized()!=null){
    		    pimSpec.setLocalized(spec.isLocalized());
    		}
            
            //for the primary key
            String path = spec.getName()+"/"+spec.getAttributes().get(0).getName();
            pimSpec.createAttributeDefinition(path, 0);
            
            //type            
			if(spec.getType().equals("PRIMARY_SPEC")){				         
	            ((PrimarySpec)pimSpec).setPrimaryKey(path);
			}else if(spec.getType().equals("LOOKUP_SPEC")){
				((LookupSpec)pimSpec).setPrimaryKey(path);
			}
			
			//remaing attribs
			if(spec.getAttributes().size()>1){
    			for (int i = 1; i < spec.getAttributes().size(); i++) {
    				path = spec.getName()+"/"+spec.getAttributes().get(i).getName();
    				pimSpec.createAttributeDefinition(path, i);
    			}
			}
				
            //save
            pimSpec.save();
    		
            //to return
            com.ibm.pim.spec.Spec newSpec = specManager.getSpec(spec.getName());
            retSpec = new Spec(newSpec);
    		
    	}
    	catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(context!=null)
            	context.cleanUp();
        }
    	
		return retSpec;
    }

    @WebMethod
    public Spec updateSpec(Context ctx, Spec spec)
    {
        if(ctx == null || spec == null){
            return null;
        }else if (spec.getName() == null){
            return null;
        }
    	//ServiceResponse response = new ServiceResponse(new ServiceRequest(ctx));
    	Spec retSpec = null;
    	
    	com.ibm.pim.context.Context context = null;
    	try{
    		
    		context = ctx.toPIMObject();
    		
    		// PIM context
    		com.ibm.pim.spec.SpecManager specManager = context.getSpecManager();
    		
    		// create PIM Spec.
            com.ibm.pim.spec.Spec pimSpec = specManager.getSpec(spec.getName());
            
            // set 
            if(spec.isLocalized()!=null){
                pimSpec.setLocalized(spec.isLocalized());
            }
            
            //save
            pimSpec.save();
            
            //to return 
            com.ibm.pim.spec.Spec newSpec = specManager.getSpec(spec.getName());            
            retSpec = new Spec(newSpec);
    		
    	}
    	catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(context!=null)
            	context.cleanUp();
        }
    	
        return retSpec;
    }

    @WebMethod
    public Spec getSpec(Context ctx, Spec spec)
    {
        if(ctx == null || spec == null){
            return null;
        }else if (spec.getName() == null){
            return null;
        }

    	Spec newSpec = null;
    	
    	com.ibm.pim.context.Context context = null;
    	try{
    		
    		context = ctx.toPIMObject();
    		
    		// PIM context
    		com.ibm.pim.spec.SpecManager specManager = context.getSpecManager();
    		
    		// get PIM Spec.
            //com.ibm.pim.spec.Spec pimSpec = specManager.getSpec(spec.getId());
    		com.ibm.pim.spec.Spec pimSpec = specManager.getSpec(spec.getName());
            
            // new PIM Spec.
            newSpec = new Spec(pimSpec);
    		
    	}
    	catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(context!=null)
            	context.cleanUp();
        }
        return newSpec;
    }

    @WebMethod
    public void deleteSpec(Context ctx, Spec spec)
    {
        
        if(ctx == null || spec == null){
            return;
        }else if (spec.getName() == null){
            return;
        }
    	
    	com.ibm.pim.context.Context context = null;
    	try{
    		
    		context = ctx.toPIMObject();
    		
    		// PIM context
    		com.ibm.pim.spec.SpecManager specManager = context.getSpecManager();
    		
    		// get PIM Spec.
            com.ibm.pim.spec.Spec pimSpec = specManager.getSpec(spec.getName());
            
            //delete PIM spec
            pimSpec.delete();
    		
    	}
    	catch (Exception e) 
        {
            e.printStackTrace();
        }
        finally
        {
            if(context!=null)
            	context.cleanUp();
        }

    }
}
