/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.objects;

/**
 * The class <code>Catalog</code> is a value object for catalog.
 * <p>
 *
 * @author maobing
 * @since 9.0.0
 *
 */
public class Catalog extends Container
{

    // constructor variables
    private Spec primarySpec;
    private Hierarchy primaryHierarchy;
    
    // other variables
    private Boolean isCached;
    private String versionName;
    private String[] customParameterKeys;
    private String[] customParameterValues;
    private String[] deleteCustomParameterKeys;
    private Hierarchy[] addSecondaryHierarchies;
    private Hierarchy[] removeSecondaryHierarchies;
    
    /**
     * Default constructor.
     */
    public Catalog()
    {

    }
    
    //create from variables
    public Catalog(int id, String name, Spec primarySpec, Hierarchy primaryHierarchy, 
                                                   AccessControlGroup accessControlGroup, String displayAttributeName){
        this.id = id;
        this.name= name;
        this.accessControlGroup= accessControlGroup;
        this.displayAttributeName = displayAttributeName;
        this.primarySpec = primarySpec;
        this.primaryHierarchy = primaryHierarchy;
    }
    
    // Create from a pim catalog
    public Catalog(com.ibm.pim.catalog.Catalog pimCatalog){
        this.id = pimCatalog.getId();
        this.name = pimCatalog.getName();
        this.primarySpec = new Spec( pimCatalog.getSpec() );
        this.primaryHierarchy = new Hierarchy( pimCatalog.getPrimaryHierarchy() );
        this.accessControlGroup = new AccessControlGroup( pimCatalog.getAccessControlGroup() );
        this.isCached = pimCatalog.isCached();
        this.displayAttributeName =  pimCatalog.getDisplayAttribute().getName();
    }
    
    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public Spec getPrimarySpec() {
        return primarySpec;
    }

    public void setPrimarySpec(Spec primarySpec) {
        this.primarySpec = primarySpec;
    }

    public Hierarchy getPrimaryHierarchy() {
        return primaryHierarchy;
    }

    public void setPrimaryHierarchy(Hierarchy primaryHierarchy) {
        this.primaryHierarchy = primaryHierarchy;
    }

    public Boolean isCached() {
        return isCached;
    }
    
    public void setCached(Boolean isCached) {
        this.isCached = isCached;
    }

    public String getVersionName() {
        return versionName;
    }

    public void setVersionName(String versionName) {
        this.versionName = versionName;
    }
    
    public String[] getCustomParameterKeys() {
        return customParameterKeys;
    }

    public void setCustomParameterKeys(String[] customParameterKeys) {
        this.customParameterKeys = customParameterKeys;
    }

    public String[] getCustomParameterValues() {
        return customParameterValues;
    }

    public void setCustomParameterValues(String[] customParameterValues) {
        this.customParameterValues = customParameterValues;
    }
    
    public Hierarchy[] getAddSecondaryHierarchies()
    {
        return addSecondaryHierarchies;
    }

    public void setAddSecondaryHierarchies(Hierarchy[] addSecondaryHierarchies)
    {
        this.addSecondaryHierarchies = addSecondaryHierarchies;
    }

    public Hierarchy[] getRemoveSecondaryHierarchies()
    {
        return removeSecondaryHierarchies;
    }

    public void setRemoveSecondaryHierarchies(Hierarchy[] removeSecondaryHierarchies)
    {
        this.removeSecondaryHierarchies = removeSecondaryHierarchies;
    }

    public String[] getDeleteCustomParameterKeys()
    {
        return deleteCustomParameterKeys;
    }

    public void setDeleteCustomParameterKeys(String[] deleteCustomParameterKeys)
    {
        this.deleteCustomParameterKeys = deleteCustomParameterKeys;
    }
    
}