/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.objects;



import com.ibm.pim.job.Executable.Type;

/**
 * The class <code>Job</code> is a value object for Job.
 * <p>
 *
 * @author maobing, uttama
 * @since 9.0.0
 *
 */
public class Job
{
    /**
     * Default Job id.
     */
    public static final int INVALID_ID = -1;
    
    /**
     * Default Job type.
     */
    public static final int UNKNOWN_TYPE = -1;    
    /**
     * Type for Report job.
     */
    public static final int REPORT = 0;
    
    protected int id;
    protected String name;
    protected int type;
    protected String description;    
   

    /**
     * Default constructor.    
     */
    public Job()
    {
        this(INVALID_ID, null, null);
    }

    /**
     * Constructor a Job object.
     * <p>
     * @param id
     * @param name
     * @param type
     */
    public Job(int id, String name, String description)
    {
        this.id = id;
        this.name = name;
        this.description = description;

        this.type = UNKNOWN_TYPE;
    }
    
    /**
     * Constructor a Job object from PIM object.
     * <p>
     * @param pimJob the PIM job object.
     */
    public Job(com.ibm.pim.job.Job pimJob)
    {
        this.id = INVALID_ID;   
        this.description = pimJob.getDescription(); 
        this.name = pimJob.getDescription();
        
        this.type = getType(pimJob);        
    }
    
    // Get job type from PIM Object.
    private int getType(com.ibm.pim.job.Job pimJob)
    {
        com.ibm.pim.job.Executable executable = pimJob.getExecutable();
        Type type = executable.getType();
        if (type == com.ibm.pim.job.Executable.Type.REPORT)
        {
            return REPORT;
        }
        
        return UNKNOWN_TYPE;
    }

    /**
     * @return Returns the id.
     */
    public int getId()
    {
        return id;
    }

    /**
     * @param id The id to set.
     */
    public void setId(int id)
    {
        this.id = id;
    }

    /**
     * @return Returns the name.
     */
    public String getName()
    {
        return name;
    }

    /**
     * @param name The name to set.
     */
    public void setName(String name)
    {
        this.name = name;
    }

    /**
     * @return Returns the type.
     */
    public int getType()
    {
        return type;
    }

    /**
     * @param type The type to set.
     */
    public void setType(int type)
    {
        this.type = type;
    }

    /**
     * @return Returns the description.
     */
    public String getDescription()
    {
        return description;
    }

    /**
     * @param description The description to set.
     */
    public void setDescription(String description)
    {
        this.description = description;
    }
}
