/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/

package com.ibm.pim.services.objects;

import java.util.Date;


/**
 * The class <code>Schedule</code> is a value object for schedule.
 * <p>
 *
 * @author maobing, uttama
 * @since 9.0.0
 *
 */
public class Schedule
{
    /**
     * Default Schedule id.
     */
    public static final int INVALID_ID = -1;
    
    /**
     * Default Schedule type.
     */
    public static final int UNKNOWN_TYPE    = -1;
    /**
     * Schedule type for job scheduled to be run immediately.
     */
    public static final int IMMEDIATE       = 0;
    /**
     * Schedule type for job to be run one time.
     */
    public static final int ONE_TIME        = 1;
    /**
     * Schedule type for job to be run at minute intervals.
     */
    public static final int MINUTE          = 2;
    /**
     * Schedule type for job to be run hourly.
     */
    public static final int HOURLY          = 3;
    /**
     * Schedule type for job to be run daily.
     */
    public static final int DAILY           = 4;
    /**
     * Schedule type for job to be run weekly.
     */
    public static final int WEEKLY          = 5;
    /**
     * Schedule type for job to be run monthly.
     */
    public static final int MONTHLY         = 6;
    /**
     * Schedule type for job to be run yearly.
     */
    public static final int YEARLY          = 7;
    
    private int id;
    private String name;
    private int type;
    private String description;
    private Job job;
    private Date startTime;
    private int intervalMinutes;

    /**
     * Default constructor.    
     */
    public Schedule()
    {
        this.id = INVALID_ID;
        this.name = null;
        this.type = -1;
        this.job = null;
    }
    
    /**
     * Construct a Schedule object.
     * <p>
     * @param id
     * @param name
     * @param type
     */
    public Schedule(int id, String name, int type, Job job)
    {
        this.id = id;
        this.name = name;
        this.type = type;
        this.job = job;
    }
    
    /**
     * Construct a Job object from PIM object.
     * <p>
     * @param pimJob the PIM job object.
     */
    public Schedule(com.ibm.pim.job.Schedule pimSchedule)
    {
        this.id = INVALID_ID; 
        this.name = pimSchedule.getName();
        this.type = getType(pimSchedule);
        this.description = pimSchedule.getName(); 
        this.job = new Job(pimSchedule.getJob());
        this.startTime = pimSchedule.getStartTime();
    }
    
    // Get Schedule type from PIM object.
    private int getType(com.ibm.pim.job.Schedule pimSchedule)
    {
        com.ibm.pim.job.Schedule.Type type = pimSchedule.getType();
        if (type == com.ibm.pim.job.Schedule.Type.DAILY)
        {
            return DAILY;
        } 
        else if (type == com.ibm.pim.job.Schedule.Type.HOURLY)
        {
            return HOURLY;
        }
        else if (type == com.ibm.pim.job.Schedule.Type.IMMEDIATE)
        {
            return IMMEDIATE;
        }
        else if (type == com.ibm.pim.job.Schedule.Type.MINUTE)
        {
            return MINUTE;
        }
        else if (type == com.ibm.pim.job.Schedule.Type.MONTHLY)
        {
            return MONTHLY;
        }
        else if (type == com.ibm.pim.job.Schedule.Type.ONE_TIME)
        {
            return ONE_TIME;
        }
        else if (type == com.ibm.pim.job.Schedule.Type.WEEKLY)
        {
            return WEEKLY;
        }
        else if (type == com.ibm.pim.job.Schedule.Type.YEARLY)
        {
            return YEARLY;
        }        
        
        return UNKNOWN_TYPE;
    }    
    
    /**
     * Compare service value object to the given PIM object.
     * <P>
     * @param pimSchedule
     * @return
     */
    public boolean equals(com.ibm.pim.job.Schedule pimSchedule)
    {
        return pimSchedule.getName().equals(this.getName()) &&               
               pimSchedule.getStartTime().compareTo(this.getStartTime())==0;     
    }

    /**
     * @return Returns the id.
     */
    public int getId()
    {
        return id;
    }

    /**
     * @param id The id to set.
     */
    public void setId(int id)
    {
        this.id = id;
    }

    /**
     * @return Returns the name.
     */
    public String getName()
    {
        return name;
    }

    /**
     * @param name The name to set.
     */
    public void setName(String name)
    {
        this.name = name;
    }

    /**
     * @return Returns the type.
     */
    public int getType()
    {
        return type;
    }

    /**
     * @param type The type to set.
     */
    public void setType(int type)
    {
        this.type = type;
    }

    /**
     * @return Returns the description.
     */
    public String getDescription()
    {
        return description;
    }

    /**
     * @param description The description to set.
     */
    public void setDescription(String description)
    {
        this.description = description;
    }

    /**
     * @return Returns the job.
     */
    public Job getJob()
    {
        return job;
    }

    /**
     * @param job The job to set.
     */
    public void setJob(Job job)
    {
        this.job = job;
    }

    /**
     * @return Returns the startTime.
     */
    public Date getStartTime()
    {
        return startTime;
    }

    /**
     * @param startTime The startTime to set.
     */
    public void setStartTime(Date startTime)
    {
        this.startTime = startTime;
    }

    /**
     * @return Returns the intervalMinutes.
     */
    public int getIntervalMinutes()
    {
        return intervalMinutes;
    }

    /**
     * @param intervalMinutes The intervalMinutes to set.
     */
    public void setIntervalMinutes(int intervalMinutes)
    {
        this.intervalMinutes = intervalMinutes;
    }    
}
