/*********************************************** {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5724-V51
 *
 * (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ************************************************ {COPYRIGHT-END} **/
package com.ibm.pim.services.tester;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.Reader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;

import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.i18n.Messages;
import org.apache.axis.message.SOAPBodyElement;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.soap.SOAPConstants;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

//import com.ibm.ccd.common.error.AustinException;
//import com.ibm.ccd.common.util.AustinNode;
//import com.ibm.ccd.common.util.XMLUtils;

/**
 * @author maobing, mohana
 * This stand-alone java program invokes a WebService, gives the endpoint URL and the request test file.
 */
public class ServiceTester
{
	private static final String REQUEST_TAG = "request";
	private static final String RESPONSE_TAG = "response";
    
	public ServiceTester()
    {
        //NOOP
    }
    
    class ServiceTesterArgs 
    {
        private static final String MARK_SUITE = "suite=";
        private static final String MARK_FILE = "file=";
        String endpointURL = null;
        List<String> tests = new ArrayList<String>();
        String logDir = null;
        boolean append = false;

        ServiceTesterArgs(String[] args)
        {
            processArguments(args);
        }
        
        //process input arguments and setup out
        private void processArguments(String[] args)
        {
            String currentArgument = null;
            for (int i = 0; i < args.length; i++)
            {
                currentArgument = args[i].toLowerCase();
                if (currentArgument.startsWith("-endpoint"))
                {
                    int index = args[i].indexOf('=');
                    endpointURL = args[i].substring(index + 1).trim();
                }
                else if (currentArgument.startsWith("-file"))
                {
                    int index = args[i].indexOf('=');
                    String requestFilePath = args[i].substring(index + 1).trim();
                    if (requestFilePath != null)
                    {
                        tests.add(MARK_FILE+requestFilePath);
                    }
                }
                else if (currentArgument.startsWith("-suite"))
                {
                    int index = args[i].indexOf('=');
                    String requestSuiteFilePath = args[i].substring(index + 1).trim();
                    if (requestSuiteFilePath != null)
                    {
                        tests.add(MARK_SUITE+requestSuiteFilePath);
                    }
                }
                else if (currentArgument.startsWith("-logdir"))
                {
                    int index = args[i].indexOf('=');
                    logDir = args[i].substring(index + 1).trim();
                } 
                else if (currentArgument.startsWith("-append"))
                {
                    int index = args[i].indexOf('=');
                    if (index <= 0 || !currentArgument.endsWith("false"))
                    {
                        append = true;
                    }
                }
            }
            
            //If logdir is not provided, by default, report will be <curDir>/logs/webservice_tests.log
            if (logDir == null) {
                String curDir = new File ("").getAbsolutePath();
                logDir = new File(curDir + "\\logs\\" ).getAbsolutePath();
            }
            
            File logFolder = new File(logDir);
            if(!logFolder.exists()){
                logFolder.mkdir();
            }
        }
        
        boolean validate()
        {
            return (endpointURL != null && endpointURL.length() > 0)   // endpoint is required
                    && tests.size() > 0;                                 // must have at least one file or one suite
        }
    }
    
    class ServiceTesterResult
    {
        static final String RESULT_PASSED = "PASSED";
        static final String RESULT_FAILED = "FAILED";
        
        String status = RESULT_PASSED;
        String test = null;
        String description = null;
        String error = null;
        
        ServiceTesterResult(String status, String test, String description, String error)
        {
            this.status = status;
            this.test = test;
            this.description = description;
            this.error = error;
        }
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {
        ServiceTester tester = new ServiceTester();
        
        // process command line arguments
        ServiceTesterArgs serviceTesterArgs = tester.new ServiceTesterArgs(args);
        if (!serviceTesterArgs.validate())
        {
            System.out.println("Usage: ~ -endpoint=<endpoint URL> -file=<request filename> -suite<index file path> -logdir=<logs folder> ");
            System.out.println("-endpoint: End point URL for the WebService. Ex: http://localhost:9084/testWS/services/CatalogTestCaseWS");
            System.out.println("-file: Test case input file. To run multiple testcases in a batch, use the -suite option. Use either -file or -suite.");
            System.out.println("-suite: Path to an index file, which contains a set of testcases to be run as a suite. Use either -file or -suite.");
            System.out.println("-logdir: Full path to the directory where logs should be stored. Ex: c:/test/mylogs. Optional parameter, if not provided, logs are stored at <curDir>/logs/webservice_tests.log");
            
            //Examples:
            //1. Arguments to run based on a suite file:
            	//-endpoint=http://localhost:9090/testWS/services/CatalogTestCaseWS -suite=C:\prj\qa\olympics\webservice\testData\Catalog\Catalog.xml
            //2. Arguments to run for a specific test data file:
            	//-endpoint=http://localhost:9090/testWS/services/CatalogTestCaseWS -file=C:\prj\qa\olympics\webservice\testData\Catalog\JACT0007GetCatalog.xml
            
            System.exit(-1);
        }
        
        // default log
        PrintStream defaultLog = System.out;
        if (serviceTesterArgs.logDir != null)
        {
            File file = new File(serviceTesterArgs.logDir, "default.log");
            try
            {
                defaultLog = new PrintStream(new FileOutputStream(file, serviceTesterArgs.append), false);
            }
            catch (FileNotFoundException e)
            {
                defaultLog.println("Cannot create/open default log file: " + file.getAbsolutePath());
                System.exit(-2);
            }
        }

		// run test
        List<ServiceTesterResult> results = new ArrayList<ServiceTesterResult>();
        try
        {
            defaultLog.println("\nTest started:");
            defaultLog.println("==================================================");
            
            for (String test: serviceTesterArgs.tests)
            {
                if (test != null && test.length() > 0)
                {
                    if (test.startsWith(ServiceTesterArgs.MARK_SUITE))
                    {
                        List<ServiceTesterResult> rets = tester.runTestSuite(serviceTesterArgs.endpointURL, test.substring(ServiceTesterArgs.MARK_SUITE.length()), serviceTesterArgs.logDir, serviceTesterArgs.append);
                        results.addAll(rets);
                    }
                    else if (test.startsWith(ServiceTesterArgs.MARK_FILE))
                    {
                        ServiceTesterResult ret = tester.runTestFile(serviceTesterArgs.endpointURL, test.substring(ServiceTesterArgs.MARK_FILE.length()), defaultLog);
                        results.add(ret);
                    }
                }
            }
            
        }
        catch (AxisFault af)
        {
            defaultLog.println("\nSOAP call was UNSUCCESSFUL.");
            QName faultCode = af.getFaultCode();
            if (faultCode != null)
            {
                defaultLog.println("Fault code namespace was      '" + faultCode.getNamespaceURI() + "'\n");
                defaultLog.println("Fault code name was           '" + faultCode.getLocalPart() + "'\n");
            }
            defaultLog.println("Fault string was              '" + af.getFaultString() + "'\n");
            String actor = af.getFaultActor();
            if (actor != null)
            {
                defaultLog.println("Fault actor is                '" + af.getFaultActor() + "'\n");
            }
            af.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            defaultLog.println(e.getMessage());
        }
        
        //test summary
        createSummaryReport(serviceTesterArgs, defaultLog, results);

        
        //close default log
        defaultLog.close();
    }

    private static void createSummaryReport(ServiceTesterArgs serviceTesterArgs, PrintStream defaultLog, List<ServiceTesterResult> results)
    {
        PrintStream summaryLog = System.out;
        if (serviceTesterArgs.logDir != null)
        {
            File file = new File(serviceTesterArgs.logDir, "summary.log");
            try
            {
                summaryLog = new PrintStream(new FileOutputStream(file, serviceTesterArgs.append), false);
            }
            catch (FileNotFoundException e)
            {
                defaultLog.println("Cannot create/open default log file: " + file.getAbsolutePath());
                System.exit(-2);
            }
        }
        
        int tcPassedCount = 0;
        int tcFailedCount = 0;

        summaryLog.println("\nTest summary:");
        summaryLog.println("==================================================");
        summaryLog.println("Total number of tests run:" + results.size());
        for (ServiceTesterResult ret:results)
        {
            if (ret.status.equalsIgnoreCase(ServiceTesterResult.RESULT_PASSED))
                ++tcPassedCount;
            else if (ret.status.equalsIgnoreCase(ServiceTesterResult.RESULT_FAILED))
                ++tcFailedCount;
            
            // more todo
        }
        summaryLog.println("Total number of passed test cases:" + tcPassedCount);
        summaryLog.println("Total number of failed test cases:" + tcFailedCount);
        summaryLog.println("==================================================");
            
        summaryLog.println("Test completed!");
        summaryLog.close();
    }
    
    private List<String> readIndexFile(String requestSuiteFilePaths) throws IOException
    {
        List<String> list = new ArrayList<String>();
        BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(requestSuiteFilePaths)));
        String line = null;
        while((line = reader.readLine()) != null)
        {
            line = line.trim();
            if (line.length() > 0)
            {
                if (!new File(line).isAbsolute())
                {
                    line = new File(new File(requestSuiteFilePaths).getParent(), line).getAbsolutePath();
                }
                list.add(line);
            }
        }
        return list;
    }

    //run a single test or all tests in a folder
    private List<ServiceTesterResult> runTestSuite(String url, String filePath, String logDir, boolean appendLog) throws IOException
    {
        String fileName = new File(filePath).getName();
        int lastIndexOf = fileName.lastIndexOf(".");
        if (lastIndexOf > 0)
        {
            fileName = fileName.substring(0, lastIndexOf);
        }
        fileName = fileName + ".log";
        File logFilePath = new File(logDir, fileName);
        PrintStream out = new PrintStream(new FileOutputStream(logFilePath, appendLog), false);
        
    	List<ServiceTesterResult> rets = new ArrayList<ServiceTesterResult>();
        List<String> filePaths = readIndexFile(filePath);
    	
        for (String path:filePaths)
        {
            ServiceTesterResult ret = runTestFile(url, path, out);
            rets.add(ret);
        }
        
        out.close();
        
        return rets;
    }
    
    private ServiceTesterResult runTestFile(String url, String filePath, PrintStream out)
    {
        File file = new File(filePath);
        
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String curTime = dateFormat.format(Calendar.getInstance().getTime());

        out.println("===========================================================================================================");
    	out.println(curTime + ": Testcase: " + file.toString());
		out.println("-----------------------------------------------------------------------------------------------------------");
    	
        //1. retrieve request data from test data
        //Sample request: String requestData = "  <ibm:getStockQuote xmlns:ibm=\"http://ibm.com/wpc/test/stockQuote\"><ticker>IBM</ticker></ibm:getStockQuote>";
        String requestData;
        try
        {
            requestData = buildXMLString(file, REQUEST_TAG);
            requestData = requestData.replaceAll("<request>\n", "");
            requestData = requestData.replaceAll("</request>\n", "");               
            out.println("Request XML:\n" + requestData);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            String ret = "Failed - Request XML:\n" + e.getMessage();
            out.println(ret);
            return new ServiceTesterResult(ServiceTesterResult.RESULT_FAILED, "", "", "");
        }
    	
        //2. retrieve expected response value from test data
        String expectedResult = "";
        try
        {
            expectedResult = buildXMLStringForComparison(file, RESPONSE_TAG);
            expectedResult = expectedResult.replaceAll("<response>", "");
            expectedResult = expectedResult.replaceAll("</response>\n", "");
            expectedResult = expectedResult.replaceAll("</response>", "");
            expectedResult = expectedResult.replaceAll("<response/>\n", "");
            expectedResult = expectedResult.replaceAll("<response/>", "").trim();
            expectedResult = getExpectedData(expectedResult);
            out.println("Expected Response Value:\n" + expectedResult);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            String ret = "Failed - Expected Response Value: " + e.getMessage();
            out.println(ret);
            return new ServiceTesterResult(ServiceTesterResult.RESULT_FAILED, filePath, "", ret);
        }
    	
        //3. prepare service call
        Service service = new Service();
        Call call;
        try
        {
            call = (Call) service.createCall();
            call.setTargetEndpointAddress(new java.net.URL(url));
        }
        catch (ServiceException se)
        {
        	String ret = "\nUnable to create Call object due to : " + se.getClass();
            out.println(ret);
            return new ServiceTesterResult(ServiceTesterResult.RESULT_FAILED, filePath, "", ret);
        }
        catch (MalformedURLException mue)
        {
        	String ret = "\nBad web service URL due to : " + mue.getClass();
            out.println(ret);
            return new ServiceTesterResult(ServiceTesterResult.RESULT_FAILED, filePath, "", ret);
        }
        
        call.setUsername(null);
        call.setPassword(null);
        String wsResponseObj = null;
        try
        {
            wsResponseObj = invoke(call, requestData.getBytes(), true);
        }
        catch (AxisFault e)
        {
            e.printStackTrace();
        	String ret = "\nUnable to invoke a service Call due to exception: " + e.getClass();
            out.println(ret);
            return new ServiceTesterResult(ServiceTesterResult.RESULT_FAILED, filePath, "", ret);
        }
        
        //4. compare result obj with expected result value
        String receivedResult = getReceivedResult(out, wsResponseObj);
        //receivedResult = cleanReceivedResult(receivedResult, expectedResult);
        
        //are the values equal?
        boolean ret = receivedResult != null?receivedResult.equalsIgnoreCase(expectedResult):false;
        //out.println(receivedResult+"\n"+expectedResult+"\n");
		out.println("\nSOAP call was successful.");
		out.println("Result was                    '" + (ret?ServiceTesterResult.RESULT_PASSED:ServiceTesterResult.RESULT_FAILED) + "'");
        
        return (ret?new ServiceTesterResult(ServiceTesterResult.RESULT_PASSED, filePath, "", "")
                   :new ServiceTesterResult(ServiceTesterResult.RESULT_FAILED, filePath, "", ""));
    }

    /*result format: 
    <p427:testGetAllCatalogsResponse xmlns:p427="http://ws.ccd.ibm.com">
        <testGetAllCatalogsReturn>ctg catalog1;ctg catalog2;ctg catalog3;ctg catalog4;ctg catalog5;ctg catalog6;processing catalog;</testGetAllCatalogsReturn>
    </p427:testGetAllCatalogsResponse> 
    */
    private String getReceivedResult(PrintStream out, String wsResponseObj)
    {
        int start, end = 0;
        start = wsResponseObj.indexOf("<return/>");			    
        if (start != -1) {start += 9;}							//8 => go until end of 'Return>'
        if (start == -1)
        {
            start = wsResponseObj.indexOf("<return>");			
            if (start != -1) {start += 8;}						//7 => go until end of 'Return>'
        }
    	end = wsResponseObj.indexOf("</return>");
        String receivedResult = "";
    	if (start != -1 && end != -1) 
        {
	        receivedResult = wsResponseObj.substring(start, end);
	        out.println("\nActual Response value:\n" + receivedResult);
    	}
        out.println("\nFull response from WebService call:\n" + wsResponseObj);
        return receivedResult;
    }

//    private String cleanReceivedResult(String rcvdRes,String expectedRes)
//    {
//    	String receivedResult = rcvdRes;
//		String amp = "&";
//		String lt = "<";
//		String gt = ">";
//		String quot = "\"";
//		String apos = "'";
//		String escapeamp = "&amp;";
//		String escapelt = "&lt;";
//		String escapegt = "&gt;";
//		String escapequot = "&quot;";
//		String escapeapos = "&apos;";
//		
//		if (receivedResult != "" && receivedResult != null && receivedResult != "\n"){
//			receivedResult = receivedResult.replaceAll(escapeamp, Pattern.quote(amp));
//			receivedResult = receivedResult.replaceAll(escapelt, Pattern.quote(lt));
//			receivedResult = receivedResult.replaceAll(escapegt, Pattern.quote(gt));
//			receivedResult = receivedResult.replaceAll(escapequot, Pattern.quote(quot));
//			receivedResult = receivedResult.replaceAll(escapeapos, Pattern.quote(apos));
//		}
//    	
//    	//If expected result text is in CDATA block then received result needs to be in CDATA block. 
//        if(expectedRes.contains("<![CDATA")) receivedResult = "<![CDATA[" + receivedResult + "]]>";
//
//        return receivedResult;
//    }
    
    private String buildXMLString(File xmlFilePath, String tag) throws SAXException, IOException
    {
    	String xmlString = "";
    	
	    Reader reader = new FileReader(xmlFilePath);
	    Document doc = read(reader, false, false);
	    NodeList elements = doc.getElementsByTagName(tag);
	    	    
	    for (int index = 0; index < elements.getLength(); index++)
	    {
	        Node item = elements.item(index);
	        if (item != null) {
	        	xmlString = elementToString(item);
	        }
	    }
	    reader.close();
	    return xmlString;  
    }
    
    private String buildXMLStringForComparison(File xmlFilePath, String tag) throws SAXException, IOException
    {
        String xmlString = "";
        
        Reader reader = new FileReader(xmlFilePath);
        Document doc = read(reader, false, false);
        NodeList elements = doc.getElementsByTagName(tag);
                
        for (int index = 0; index < elements.getLength(); index++)
        {
            Node item = elements.item(index);
            if (item != null) {
                xmlString = elementToStringForComparison(item);
            }
        }
        reader.close();
        return xmlString;  
    }
    
    private String getExpectedData(String expectedResult)
    {
    	String expectedRes = expectedResult;
		//String amp = "&";
		String lt = "<";
		String gt = ">";
		String quot = "\"";
		//String apos = "'";
		//String escapeamp = "&amp;";
		String escapelt = "&lt;";
		String escapegt = "&gt;";
		String escapequot = "&quot;";
		//String escapeapos = "&apos;";
 
    	if (expectedRes != null)
    	{
    		expectedRes = expectedRes.trim();
    		int index = expectedRes.indexOf("<![CDATA[");
    		int indexEnds = expectedRes.lastIndexOf("]]");
    		if (index >= 0)
    		{
    			if (indexEnds >= 0)
    			{
    				expectedRes = expectedRes.substring(index+"<![CDATA[".length(), indexEnds);
    			}else {
    				expectedRes = expectedRes.substring(index+"<![CDATA[".length());
    			}
        		if (expectedRes != "" && expectedRes != null && expectedRes != "\n"){
        			//expectedRes = expectedRes.replaceAll(Pattern.quote(amp), escapeamp);
        			expectedRes = expectedRes.replaceAll(Pattern.quote(lt), escapelt);
        			expectedRes = expectedRes.replaceAll(Pattern.quote(gt), escapegt);
        			expectedRes = expectedRes.replaceAll(Pattern.quote(quot), escapequot);
        			//expectedRes = expectedRes.replaceAll(apos, escapeapos);
        		}
    		}
    	}
    	return expectedRes;
    }
    
    private String elementToString(Node node) throws IOException
    {
	    Element ele = (Element) node;
	    StringWriter sw = new StringWriter();
	
	    OutputFormat format = new OutputFormat();
	    format.setIndenting(true);
	    format.setLineWidth(0);
	    format.setOmitXMLDeclaration(true);
	
	    XMLSerializer serializer = new XMLSerializer(format);
	    serializer.setOutputCharStream(sw);
        serializer.serialize(ele);

	    return sw.toString();
    }
    
    private String elementToStringForComparison(Node node) throws IOException
    {
        Element ele = (Element) node;
        StringWriter sw = new StringWriter();
    
        OutputFormat format = new OutputFormat();
        format.setIndenting(false);
        format.setLineWidth(0);
        format.setOmitXMLDeclaration(true);
    
        XMLSerializer serializer = new XMLSerializer(format);
        serializer.setOutputCharStream(sw);
        serializer.serialize(ele);
        
        String sRet = sw.toString().replaceAll("\n", "").replaceAll("\t", "");
        
        return sRet;
    }

	
    private String invoke(Call call, final byte[] message, final boolean bodyInStream) throws AxisFault
    {
        try
        {
            InputStream fis = new ByteArrayInputStream(message);
            SOAPEnvelope env;
            if (bodyInStream)
            {
                SOAPConstants soapConsts = call.getMessageContext().getSOAPConstants();
                env = new SOAPEnvelope(soapConsts, call.getMessageContext().getSchemaVersion());

                SOAPBodyElement elmt = new SOAPBodyElement(fis);
                env.addBodyElement(elmt);
            }
            else
            {
                env = new SOAPEnvelope(fis);
            }

            Message msg = new Message(env, bodyInStream);

            call.setRequestMessage(msg);
            call.invoke();

            msg = call.getMessageContext().getResponseMessage();
            if (msg == null)
            {
                if (call.getMessageContext().isPropertyTrue(Call.FAULT_ON_NO_RESPONSE, false))
                {
                    throw new AxisFault(Messages.getMessage("nullResponse00"));
                }
                else
                {
                    return null;
                }
            }

            env = msg.getSOAPEnvelope();
            @SuppressWarnings("rawtypes")
			Vector v = env.getBodyElements();
            StringBuffer buff = new StringBuffer();
            for (int i = 0; i < v.size(); i++)
            {
                buff.append(((SOAPBodyElement) v.get(i)).toString());
            }
            return buff.toString();
        }
        catch (SAXException e)
        {
            throw new AxisFault(e.getMessage());
        }
    }

    public static Document read(Reader reader,
            boolean validate,
            boolean disableNamespaces) throws SAXException, IOException
	{
		DOMParser parser = new DOMParser();
		ErrorHandler handler = new ErrorHandler() 
		{
		    public StringBuffer sbWarning = new StringBuffer(1024);
		    public StringBuffer sbError = new StringBuffer(1024);
		    public StringBuffer sbFatal = new StringBuffer(1024);
		    
		    /**
		     * Report all warnings, and continue parsing.
		     */
		    public void warning(SAXParseException exception)
		    {
		        sbWarning.append("\nWarning: " +
		            exception.getMessage() +
		            " (L:" +
		            exception.getLineNumber() +
		            ",C:" +
		            exception.getColumnNumber() +
		            ')');
		    }

		    /**
		     * Report all recoverable errors, and try to continue parsing.
		     */
		    public void error(SAXParseException exception)
		    {
		        sbError.append("\nRecoverable Error: " +
		            exception.getMessage() +
		            " (L:" +
		            exception.getLineNumber() +
		            ",C:" +
		            exception.getColumnNumber() +
		            ')');
		    }

		    /**
		     * Report all fatal errors, and try to continue parsing.
		     *
		     * <p>Note: results are no longer reliable once a fatal error has
		     * been reported.</p>
		     *
		     * @see org.xml.sax.ErrorHandler#fatalError
		     */
		    public void fatalError(SAXParseException exception)
		    {
		        sbFatal.append("\nFatal Error: " +
		            exception.getMessage() +
		            " (L:" +
		            exception.getLineNumber() +
		            ",C:" +
		            exception.getColumnNumber() +
		            ')');

		    }
		};
		
		parser.setErrorHandler(handler);
		if (disableNamespaces)
		{
			parser.setFeature("http://xml.org/sax/features/namespaces", false);
		}
		
		parser.setFeature("http://xml.org/sax/features/validation", validate);
		parser.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd",
		          validate);
		parser.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar",
		          validate);
		
		parser.parse(new InputSource(reader));
				
		return parser.getDocument();
	}

}
