/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5725-E59
*
* (c) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/
package com.ibm.pim.webservices.samples;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.jws.WebMethod;
import javax.jws.soap.SOAPBinding;

import com.ibm.pim.attribute.AttributeDefinition;
import com.ibm.pim.attribute.AttributeDefinitionProperty;
import com.ibm.pim.attribute.AttributeInstance;
import com.ibm.pim.catalog.Catalog;
import com.ibm.pim.catalog.CatalogManager;
import com.ibm.pim.collection.PIMCollection;
import com.ibm.pim.common.ExtendedValidationErrors;
import com.ibm.pim.common.ValidationError;
import com.ibm.pim.common.ValidationErrors;
import com.ibm.pim.hierarchy.Hierarchy;
import com.ibm.pim.hierarchy.HierarchyManager;
import com.ibm.pim.lookuptable.LookupTable;
import com.ibm.pim.lookuptable.LookupTableEntry;
import com.ibm.pim.lookuptable.LookupTableManager;
import com.ibm.pim.organization.Role;
import com.ibm.pim.organization.User;
import com.ibm.pim.search.SearchQuery;
import com.ibm.pim.search.SearchResultSet;
import com.ibm.pim.services.common.Context;
import com.ibm.pim.services.common.ServiceRequest;
import com.ibm.pim.services.common.ServiceResponse;
import com.ibm.pim.services.objects.Attribute;
import com.ibm.pim.services.objects.Category;
import com.ibm.pim.services.objects.Item;
import com.ibm.pim.services.objects.SearchPredicate;
import com.ibm.pim.spec.SecondarySpec;
import com.ibm.pim.spec.Spec;
import com.ibm.pim.spec.SpecManager;
import com.ibm.pim.utils.TimezoneResolver;

/**
 * Sample web services
 */
@javax.jws.WebService(
    targetNamespace = "http://services.pim.ibm.com/", 
    serviceName = "SampleService", 
    portName = "SamplePort")
    
@SOAPBinding(
    style = SOAPBinding.Style.DOCUMENT, 
    use = SOAPBinding.Use.LITERAL, 
    parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public class SampleService
{
    /**
     * Attempt to get context for specified user and return user details.
     * Return displayName, roles for the user, or ERROR: message.
     */
    @WebMethod
    public ServiceResponse login(Context ctx)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> loginResults = new ArrayList<String>();
        try
        {
            // login with PIM context
            pimContext = ctx.toPIMObject();
                       
            // user name
            User u = pimContext.getCurrentUser();
            loginResults.add(u.getFirstName() + " " + u.getLastName());
            
            // user roles
            StringBuffer rolesString = new StringBuffer("");
            for (Iterator<Role> iter = u.getRoles().iterator(); iter.hasNext();)
            {
                Role role = iter.next();
                if (rolesString.length() <= 0)
                {
                    rolesString.append(",");
                }
                rolesString.append(role.getName());
            }
            loginResults.add(rolesString.toString());
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception re)
        {
            loginResults.add("ERROR:" + re.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(loginResults);
        return response;
    }

    /**
     * Get all catalog names for the company
     */
    @WebMethod
    public ServiceResponse getAllCatalogs(Context ctx)
    {
        return getAllCatalogsUsingSpec(ctx, null);
    }

    /**
     * Get all catalog names for the company that use the given spec
     */
    @WebMethod
    public ServiceResponse getAllCatalogsUsingSpec(Context ctx, String specName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Collection<Catalog> ctgs = catalogManager.getCatalogs();

            for (Catalog ctg : ctgs)
            {
                if (specName == null || ctg.getSpec().getName().equals(specName))
                {
                    results.add(ctg.getName());
                }
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get all lookup table names for the company using the given spec
     */
    @WebMethod
    public ServiceResponse getAllLookupTables(Context ctx)
    {
        return getAllLookupTablesUsingSpec(ctx, null);
    }

    /**
     * Get all lookup table names for the company using the given spec
     */
    @WebMethod
    public ServiceResponse getAllLookupTablesUsingSpec(Context ctx, String specName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            LookupTableManager lookupManager = pimContext.getLookupTableManager();
            Collection<LookupTable> lkups = lookupManager.getLookupTables();

            for (LookupTable lkup : lkups)
            {
                if (specName == null || lkup.getSpec().getName().equals(specName))
                {
                    results.add(lkup.getName());
                }
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get all Item keys for the given catalog
     */
    @WebMethod
    public ServiceResponse getAllItemsInCatalog(Context ctx, String catalogName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.getAllItemsInCatalog: " + catalogName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(catalogName);
            PIMCollection<com.ibm.pim.catalog.item.Item> items = ctg.getItems();
            String keyFieldPath = ctg.getSpec().getPrimaryKeyAttributeDefinition().getPath();
            for (com.ibm.pim.catalog.item.Item item : items)
            {
                results.add((String)item.getAttributeValue(keyFieldPath));
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get all entries in the given lookup table
     */
    @WebMethod
    public ServiceResponse getAllEntriesInLookupTable(Context ctx, String lookupName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.getAllEntriesInLookupTable: " + lookupName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            LookupTableManager lookupManager = pimContext.getLookupTableManager();
            LookupTable lkup = lookupManager.getLookupTable(lookupName);

            PIMCollection<LookupTableEntry> entries = lkup.getLookupTableEntries();
            String keyFieldPath = lkup.getSpec().getPrimaryKeyAttributeDefinition().getPath();
            for (LookupTableEntry entry : entries)
            {
                results.add((String) entry.getAttributeValue(keyFieldPath));
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get all the categories available within the given catalog.
     * Return a list of category paths.
     */
    @WebMethod
    public ServiceResponse getCategoriesForCatalog(Context ctx, String catalogName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.getCategoriesForCatalog: " + catalogName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(catalogName);
            ArrayList<Hierarchy> hiers = new ArrayList<Hierarchy>();
            hiers.add(ctg.getPrimaryHierarchy());
            hiers.addAll(ctg.getSecondaryHierarchies());
            for (Hierarchy hier : hiers)
            {
                PIMCollection<com.ibm.pim.hierarchy.category.Category> cats = hier.getCategories();
                for (com.ibm.pim.hierarchy.category.Category cat : cats)
                {
                    results.addAll(cat.getFullPaths(true));
                }
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get Lookup Table entries.
     * Return a list of Lookup table entry keys.
     */
    @WebMethod
    public ServiceResponse getLookupTableValues(Context ctx, String lookupTableName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.getLookupTableValues: " + lookupTableName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            LookupTableManager lookupManager = pimContext.getLookupTableManager();
            LookupTable lkp = lookupManager.getLookupTable(lookupTableName);
            PIMCollection<LookupTableEntry> lkpEntries = lkp.getLookupTableEntries();
            results.add("-NONE-");
            for (LookupTableEntry lkpEntry : lkpEntries)
            {
                results.add(lkpEntry.getKey());
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get the list of supported Timezone values.
     */
    @WebMethod
    public ServiceResponse getTimezoneValues(Context ctx)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            TimezoneResolver tzr = pimContext.getTimezoneResolver();
            String[] possValues = tzr.getTimezoneStrings();
            for (String possValue : possValues)
            {
                results.add(possValue);
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get a list of the defined values for the given Number Enumeration.
     */
    @WebMethod
    public ServiceResponse getNumberEnumValues(Context ctx, String specName, String enumAttributeName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.getNumberEnumValues for: " + enumAttributeName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            SpecManager specManager = pimContext.getSpecManager();
            Spec spec = specManager.getSpec(specName);
            AttributeDefinition ad = spec.getAttributeDefinition(spec.getName() + "/" + enumAttributeName);
            AttributeDefinitionProperty adp = ad.getProperty(AttributeDefinitionProperty.Name.NUMBER_ENUMERATION);
            int precision = Integer.valueOf(ad.getProperty(AttributeDefinitionProperty.Name.PRECISION).getValue()).intValue();
            NumberFormat nf = NumberFormat.getInstance();
            nf.setMinimumFractionDigits(precision);
            Collection<String> adpValues = adp.getValues();
            results.add("-NONE-"); // include none for null, not set value
            for (String adpValue : adpValues)
            {
                Double d = new Double(adpValue);
                results.add(nf.format(d.doubleValue()));
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get a list of the defined values for the given String Enumeration.
     */
    @WebMethod
    public ServiceResponse getStringEnumValues(Context ctx, String specName, String enumAttributeName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.getStringEnumValues for: " + enumAttributeName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            SpecManager specManager = pimContext.getSpecManager();
            Spec spec = specManager.getSpec(specName);
            AttributeDefinition ad = spec.getAttributeDefinition(spec.getName() + "/" + enumAttributeName);
            AttributeDefinitionProperty adp = ad.getProperty(AttributeDefinitionProperty.Name.STRING_ENUMERATION);
            results.add("-NONE-"); // include none for null, not set value
            results.addAll(adp.getValues());
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get all categories to which the given Item is mapped to.
     * Return a list of category paths.
     */
    @WebMethod
    public ServiceResponse getItemCategories(Context ctx, String itemPk, String catalogName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.getItemCategories for item: " + itemPk + " in Catalog: " + catalogName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(catalogName);
            com.ibm.pim.catalog.item.Item item = ctg.getItemByPrimaryKey(itemPk);

            Collection<com.ibm.pim.hierarchy.category.Category> itemCats = item.getCategories();
            for (com.ibm.pim.hierarchy.category.Category category : itemCats)
            {
                results.addAll(category.getFullPaths(true));
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get all attribute values for the given Item.
     * Return a list of attribute values (in attribute order defined in the spec),
     * where, the first string is a csv-format string of category paths Item is mapped to,
     * or null if unassigned.
     */
    @WebMethod
    public ServiceResponse getItemAttributeValues(Context ctx, String itemPk, String catalogName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(catalogName);
            com.ibm.pim.catalog.item.Item pimItem = ctg.getItemByPrimaryKey(itemPk);

            // get mapped Categories for Item
            Collection<com.ibm.pim.hierarchy.category.Category> pimCategories = pimItem.getCategories();
            List<String> itemCategoryPathList = new ArrayList<String>();
            for (com.ibm.pim.hierarchy.category.Category pimCategory : pimCategories)
            {
                itemCategoryPathList.addAll(pimCategory.getFullPaths(true));
            }
            String itemCatsString = null;
            for (String categoryPath : itemCategoryPathList)
            {
                if (itemCatsString == null)
                {
                    itemCatsString = categoryPath;
                }
                else
                {
                    itemCatsString += "," + categoryPath;
                }
            }
            // return mapped categories in fields[0]
            results.add(itemCatsString);

            // now add attribute values in spec defined order
            List<AttributeDefinition> specAttrDefs = ctg.getSpec().getAttributeDefinitions();
            for (AttributeDefinition specAttrDef : specAttrDefs)
            {
                String specAttrPath = specAttrDef.getPath();
                String attributeValue = getItemAttributeValue(pimItem, specAttrPath);
                if (attributeValue != null)
                {
                    results.add(attributeValue);
                }
            }

            // now add attribute values for mapped item secondary specs
            HierarchyManager hierManager = pimContext.getHierarchyManager();
            ArrayList<SecondarySpec> addedSecSpecs = new ArrayList<SecondarySpec>();
            for (String categoryPath : itemCategoryPathList)
            {
                String hierName = categoryPath.substring(0, categoryPath.indexOf("/"));
                String catPath = categoryPath.substring(categoryPath.indexOf("/") + 1);
                Hierarchy hier = hierManager.getHierarchy(hierName);
                com.ibm.pim.hierarchy.category.Category category = hier.getCategoryByPath(catPath);

                Collection<SecondarySpec> secSpecs = category.getItemSecondarySpecs(ctg);
                for (SecondarySpec secSpec : secSpecs)
                {
                    List<AttributeDefinition> secSpecAttrDefs = secSpec.getAttributeDefinitions();
                    for (AttributeDefinition specAttrDef : secSpecAttrDefs)
                    {
                        String specAttrPath = specAttrDef.getPath();
                        String value = null;
                        value = getItemAttributeValue(pimItem, specAttrPath);
                        if (value != null)
                        {
                            results.add(categoryPath + ":" + value);
                        }
                    }
                    addedSecSpecs.add(secSpec);
                }
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get Lookup Table value for given lookup entry key
     */
    @WebMethod
    public ServiceResponse getLookupAttributeValues(Context ctx, String lookupKey, String lookupName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            LookupTableManager lookupManager = pimContext.getLookupTableManager();
            LookupTable lkup = lookupManager.getLookupTable(lookupName);
            LookupTableEntry lkpEntry = lkup.getLookupTableEntry(lookupKey);

            List<AttributeDefinition> specAttrDefs = lkup.getSpec().getAttributeDefinitions();
            for (AttributeDefinition specAttrDef : specAttrDefs)
            {
                String specAttrPath = specAttrDef.getPath();
                String value = getLookupEntryAttributeValue(lkpEntry, specAttrPath);
                if (value != null)
                {
                    results.add(value);
                }
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get attribute values for the given Category.
     */
    @WebMethod
    public ServiceResponse getCategoryAttributeValues(Context ctx, String categoryPath, String hierarchyName)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            HierarchyManager hierManager = pimContext.getHierarchyManager();
            Hierarchy hier = hierManager.getHierarchy(hierarchyName);
            com.ibm.pim.hierarchy.category.Category pimCategory = hier.getCategoryByPath(categoryPath);
            results.add(categoryPath);
            
            List<AttributeDefinition> specAttrDefs = hier.getPrimarySpec().getAttributeDefinitions();
            for (AttributeDefinition specAttrDef : specAttrDefs)
            {
                String specAttrPath = specAttrDef.getPath();
                String value = getCategoryAttributeValue(pimCategory, specAttrPath);
                if (value != null)
                {
                    results.add(value);
                }
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Return a list of attribute values with format of specName/attributeName*attributeType*attributeValue,
     * where,the first entry contains a csv-format of string categories that item is mapped to.
     */
    @WebMethod
    public ServiceResponse getDynamicItemAttributeValues(Context ctx, String itemPk, String catalogName)
    {
        ServiceResponse response = new ServiceResponse();
        
        System.out.println("SampleService.getDynamicItemAttributeValues for item: " + itemPk + " in Catalog: " + catalogName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(catalogName);
            ArrayList<String> specNamesList = new ArrayList<String>();
            String primarySpecName = ctg.getSpec().getName();
            specNamesList.add(primarySpecName); // add primary spec first
            com.ibm.pim.catalog.item.Item item = ctg.getItemByPrimaryKey(itemPk);
            Collection<Spec> itemSpecs = item.getSpecs();

            for (Spec spec : itemSpecs)
            {
                String specName = spec.getName();
                System.out.println("itemSpec: " + specName);
                if (!specNamesList.contains(specName))
                    specNamesList.add(specName);
            }

            Collection<com.ibm.pim.hierarchy.category.Category> itemCats = item.getCategories();
            ArrayList<String> itemCatsArrayList = new ArrayList<String>();
            for (com.ibm.pim.hierarchy.category.Category category : itemCats)
            {
                for (String catPath : category.getFullPaths(true))
                {
                    Collection<SecondarySpec> catSecSpecs = category.getItemSecondarySpecs(ctg);
                    String catPathString = catPath;
                    for (SecondarySpec secSpec : catSecSpecs)
                    {
                        catPathString += "*" + secSpec.getName();
                    }
                    itemCatsArrayList.add(catPathString);
                    System.out.println("itemCats: " + catPathString);
                }
            }
            String itemCatsString = null;
            for (String categoryPath : itemCatsArrayList)
            {
                if (itemCatsString == null)
                    itemCatsString = categoryPath;
                else
                    itemCatsString += "," + categoryPath;
            }
            results.add(itemCatsString);

            for (Spec spec : itemSpecs)
            {
                String specName = spec.getName();
                System.out.println("itemSpec: " + specName);
                List<AttributeDefinition> specAttrDefs = spec.getAttributeDefinitions();
                for (AttributeDefinition specAttrDef : specAttrDefs)
                {
                    String specAttrPath = specAttrDef.getPath();
                    System.out.println("specAttrPath: " + specAttrPath);
                    String value = null;
                    value = getDynamicItemAttributeValue(item, specAttrPath);
                    if (value != null)
                    {
                        results.add(value);
                    }
                }
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Search for items in the given catalog.
     * 
     * @param catalogName
     *            catalog to search
     * @param categoryNames
     *            category names to be used in search criteria
     * @param matchAllCategories
     *            true if search matching all the specified categories, or false
     *            if any of the specified categories
     * @param attrNames
     *            attribute names to be used in search criteria
     * @param attrTypes
     *            attribute types of attributes to be used in search criteria
     * @param attrComps
     *            attribute comparators to be used in search criteria
     * @param attrValues
     *            attribute values to be used in search criteria
     * @return keys of found items
     */
    @WebMethod
    public ServiceResponse searchItemsInCatalog(Context ctx, String catalogName, List<String> categoryNames, String matchAllCategories, List<SearchPredicate> predicates)
    {
        System.out.println("SampleService.searchItemsInCatalog: " + catalogName);
        ServiceResponse response = new ServiceResponse();
        
        String queryString = "select item from catalog('" + catalogName + "') ";
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            List<String> numericTypes = Arrays.asList("CURRENCY", "INTEGER", "NUMBER", "NUMBER_ENUMERATION", "SEQUENCE");
            boolean predicateAdded = false;
            
           // search attributes
            if (!predicates.isEmpty())
            {
                queryString += (!predicateAdded)? "where ":"";
                for (SearchPredicate predicate:predicates)
                {
                    queryString += predicateAdded?" and ":"";
                    
                    // quote
                    String quotes = "'";
                    if (numericTypes.contains(predicate.getAttibuteType()))
                    {
                        quotes = ""; // no quotes around numeric or flag
                    }
                    
                    // value
                    String value = predicate.getAttibuteValue();
                    String ops = predicate.getAttibuteOp();
                    if (value.equals("-NONE-")) // null, not set value for enumerations, Flag
                    {
                        value = "null";
                        quotes = "";
                        ops = "is";
                    }
                    
                    // predicate
                    queryString += "item['" + predicate.getAttibuteName() + "'] " + ops + " " + quotes + value + quotes;
                    predicateAdded = true;
                }
            }

            // category restrictions
            if (!categoryNames.isEmpty())
            {
                boolean matchAllCat = new Boolean(matchAllCategories).booleanValue();
                queryString += (!predicateAdded)? "where ":" and ";
                String prevQueryString = queryString;
                boolean isFirst = true;
                for (String categoryName:categoryNames)
                {
                    if (!isFirst)
                    {
                        if (matchAllCat == true) // multi-categories selected - match
                        {
                            queryString += " intersect " + prevQueryString; // All
                        }
                        else
                        {
                            queryString += " union " + prevQueryString; // Any
                        }
                        
                        queryString += (!predicateAdded)?" where ":" and ";
                    	isFirst = false;
                    }
                    
                    queryString += (categoryName.equals("Unassigned"))
                        ? "item.category is null"
                        : "item.category.path = '" + categoryName + "'";
                }
                predicateAdded = true;
            }

            System.out.println("search query: " + queryString);
            pimContext = ctx.toPIMObject();
            SearchQuery query = pimContext.createSearchQuery(queryString);
            SearchResultSet resultSet = query.execute();

            System.out.println("result set size: " + resultSet.size());
            for (int j = 0; j < resultSet.size(); j++)
            {
                // boolean b = resultSet.next();
                resultSet.next();
                // System.out.println("another result row exists = " + b);
                // strA[j] = resultSet.toString();
                // System.out.println("result set toString: " + strA[j]);
                // strA[j] = resultSet.getString(1);
                // System.out.println("result set getString: " + strA[j]);
                results.add(resultSet.getItem(1).getPrimaryKey());
                // System.out.println("result set getItem: " + strA[j]);
                // strA[j] = resultSet.getString(0);
                // System.out.println("result set getString: " + strA[j]);
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Set item attribute values
     * 
     * @param itemPk
     *            - key of item to update
     * @param catalogName
     *            - catalog containing the item
     * @param newItemCatNames
     *            - names of categories to map to
     * @param attrNames
     *            - names of attributes to update
     * @param types
     *            - corresponding types of attributes to update
     * @param attrValues
     *            - corresponding values of attributes to set
     * @return String[] - array of validation error messages
     */
    @WebMethod
    public ServiceResponse setItemAttributeValues(Context ctx, Item item, List<String> mappedToCategories)
    {
        ServiceResponse response = new ServiceResponse();
        
        System.out.println("SampleService.setItemAttributeValues for item: " + item.getPrimaryKey() + " in Catalog: " + item.getContainerName());
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(item.getContainerName());
            HierarchyManager hierarchyManager = pimContext.getHierarchyManager();
            com.ibm.pim.catalog.item.Item pimItem = ctg.getItemByPrimaryKey(item.getPrimaryKey());
            if (pimItem == null)
            {
                response.setStatus(ServiceResponse.SUCCEEDED);
                return response;
            }
            String itemPrimarySpecName = ctg.getSpec().getName();

            // get current mappings
            Collection<com.ibm.pim.hierarchy.category.Category> pimCategories = pimItem.getCategories(); 
            if (!mappedToCategories.isEmpty())
            {
                List<String> itemCatNames = new ArrayList<String>(pimCategories.size());
                for (com.ibm.pim.hierarchy.category.Category pimCategory : pimCategories)
                {
                    itemCatNames.addAll(pimCategory.getFullPaths(true));
                }

                // check for new category mappings
                List<String> newItemCatNames = new ArrayList<String>();
                for (String newItemCatName : mappedToCategories)
                {
                    newItemCatNames.add(newItemCatName);
                    if (!itemCatNames.contains(newItemCatName))
                    {
                        // add the new category mapping
                        String hierName = newItemCatName.substring(0, newItemCatName.indexOf("/"));
                        Hierarchy hier = hierarchyManager.getHierarchy(hierName);
                        com.ibm.pim.hierarchy.category.Category category = hier.getCategoryByPath(newItemCatName.substring(newItemCatName.indexOf("/") + 1));
                        pimItem.mapToCategory(category);
                    }
                }
                
                // check for removed category mappings
                for (String itemCatName : itemCatNames)
                {
                    if (!newItemCatNames.contains(itemCatName))
                    {
                        // remove the category mapping
                        String hierName = itemCatName.substring(0, itemCatName.indexOf("/"));
                        Hierarchy hier = hierarchyManager.getHierarchy(hierName);
                        com.ibm.pim.hierarchy.category.Category category = hier.getCategoryByPath(itemCatName.substring(itemCatName.indexOf("/") + 1));
                        pimItem.removeFromCategory(category);
                    }
                }
            }
            else
            {
                // all category mappings removed
                for (com.ibm.pim.hierarchy.category.Category category : pimCategories)
                {
                    pimItem.removeFromCategory(category);
                }
            }

            List<SecondarySpec> secSpecs = new ArrayList<SecondarySpec>();
            for (String categoryPath : mappedToCategories)
            {
                String hierName = categoryPath.substring(0, categoryPath.indexOf("/"));
                Hierarchy hier = hierarchyManager.getHierarchy(hierName);
                com.ibm.pim.hierarchy.category.Category category = hier.getCategoryByPath(categoryPath.substring(categoryPath.indexOf("/") + 1));

                secSpecs.addAll(category.getItemSecondarySpecs(ctg));
            }

            List<String> secSpecNames = new ArrayList<String>();
            for (Iterator<SecondarySpec> iter = secSpecs.iterator(); iter.hasNext();)
            {
                SecondarySpec secSpec = (SecondarySpec) iter.next();
                secSpecNames.add(secSpec.getName());
            }

            for (Attribute attribute:item.getAttributes())
            {
                // update all primary spec values
                // and item secondary spec values where set
                String name = attribute.getName();
                if (name.startsWith(itemPrimarySpecName)
                    || (attribute.getValue() != null && secSpecNames.contains(name.substring(0, name.indexOf("/")))))
                    setItemAttributeValue(pimContext, pimItem, attribute);
            }
            ExtendedValidationErrors verrs = pimItem.save();
            List<ValidationErrors> errs = new ArrayList<ValidationErrors>();
            errs.add(verrs);
            response.setStatus(checkErrors(item.getPrimaryKey(), results, errs, "Set item attributes"));
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Set Lookup Table values
     * 
     * @param lookupKey
     *            - key of item to update
     * @param lookupName
     *            - catalog containing the item
     * @param newItemCatNames
     *            - names of categories to map to
     * @param attrNames
     *            - names of attributes to update
     * @param types
     *            - corresponding types of attributes to update
     * @param attrValues
     *            - corresponding values of attributes to set
     * @return String[] - array of validation error messages
     */
    @WebMethod
    public ServiceResponse setLookupAttributeValues(Context ctx, Item item)
    {
        ServiceResponse response = new ServiceResponse();
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            LookupTableManager lookupManager = pimContext.getLookupTableManager();
            LookupTable pimLkup = lookupManager.getLookupTable(item.getContainerName());
            String specName = pimLkup.getSpec().getName();
            LookupTableEntry entry = pimLkup.getLookupTableEntry(item.getPrimaryKey());
            if (entry == null)
            {
                response.setStatus(ServiceResponse.SUCCEEDED);
                return response;
            }
            for (Attribute attribute:item.getAttributes())
            {
                if (attribute.getName() != null)
                    setLookupEntryAttributeValue(pimContext, entry, specName, attribute);
            }
            ValidationErrors verrs = entry.save();
            List<ValidationErrors> errs = new ArrayList<ValidationErrors>();
            errs.add(verrs);
            response.setStatus(checkErrors(item.getPrimaryKey(), results, errs, "Update lookup table entry"));
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Set category attribute values
     * 
     * @param categoryPath
     *            - key of item to update
     * @param hierarchyName
     *            - catalog containing the item
     * @param newItemCatNames
     *            - names of categories to map to
     * @param attrNames
     *            - names of attributes to update
     * @param types
     *            - corresponding types of attributes to update
     * @param attrValues
     *            - corresponding values of attributes to set
     * @return String[] - array of validation error messages
     */
    @WebMethod
    public ServiceResponse setCategoryAttributeValues(Context ctx, Category category)
    {
        ServiceResponse response = new ServiceResponse();
        String categoryPath = category.getCategoryPath();
        String hierarchyName = category.getContainerName();
        
        System.out.println("SampleService.setCategoryAttributeValues for category: " + categoryPath + " in Hierarchy: "
                           + hierarchyName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            HierarchyManager hierarchyManager = pimContext.getHierarchyManager();
            Hierarchy pimHierarchy = hierarchyManager.getHierarchy(hierarchyName);
            com.ibm.pim.hierarchy.category.Category pimCategory = pimHierarchy.getCategoryByPath(categoryPath);
            String categoryPrimarySpecName = pimHierarchy.getPrimarySpec().getName();
            if (pimCategory == null)
            {
                response.setStatus(ServiceResponse.SUCCEEDED);
                return response;
            }
            for (Attribute attribute:category.getAttributes())
            {
                if (attribute.getName() != null)
                {
                    setCategoryAttributeValue(pimContext, pimCategory, categoryPrimarySpecName, attribute);
                }
            }
            List<ValidationErrors> verrs = pimHierarchy.save();
            response.setStatus(checkErrors(category.getPrimaryKey(), results, verrs, "Update category attributes"));
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Create an item with the given attribute values
     * 
     * @param itemPk
     *            - key of item to create
     * @param containerName
     *            - catalog containing the item
     * @param mappedToCategories
     *            - names of categories to map to
     * @param attrNames
     *            - names of attributes to update
     * @param types
     *            - corresponding types of attributes to update
     * @param attrValues
     *            - corresponding values of attributes to set
     * @return String[] - array of validation error messages
     */
    @WebMethod
    public ServiceResponse createItem(Context ctx, Item item, List<String> mappedToCategories)
    {
        System.out.println("SampleService.createItem: " + item.getPrimaryKey() + " in Catalog: " + item.getContainerName());
        
        ServiceResponse response = new ServiceResponse();
        
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(item.getContainerName());
            String specName = ctg.getSpec().getName();
            HierarchyManager hierarchyManager = pimContext.getHierarchyManager();
            // String specName = ctg.getSpec().getName();
            com.ibm.pim.catalog.item.Item pimItem = ctg.createItem();
            pimItem.setPrimaryKey(item.getPrimaryKey());

            List<SecondarySpec> secSpecs = new ArrayList<SecondarySpec>();

            if (!mappedToCategories.isEmpty())
            {
                // check for new category mappings
                for (String newItemCatName : mappedToCategories)
                {
                    // add the new category mapping
                    String hierName = newItemCatName.substring(0, newItemCatName.indexOf("/"));
                    // System.out.println("hier: " + hierName);
                    Hierarchy hier = hierarchyManager.getHierarchy(hierName);
                    com.ibm.pim.hierarchy.category.Category category = hier.getCategoryByPath(newItemCatName.substring(newItemCatName.indexOf("/") + 1));
                    pimItem.mapToCategory(category);
                    secSpecs.addAll(category.getItemSecondarySpecs(ctg));
                }
            }

            ArrayList<String> secSpecNames = new ArrayList<String>();
            for (Iterator<SecondarySpec> iter = secSpecs.iterator(); iter.hasNext();)
            {
                SecondarySpec secSpec = (SecondarySpec) iter.next();
                secSpecNames.add(secSpec.getName());
            }

            for (Attribute attribute:item.getAttributes())
            {
                // set all primary spec values
                // and item secondary spec values where applicable
                String name = attribute.getName();
                String value = attribute.getValue();
                if (name.startsWith(specName)
                    || (value != null && secSpecNames.contains(name.substring(0, name.indexOf("/")))))
                    setItemAttributeValue(pimContext, pimItem, attribute);
            }
            ExtendedValidationErrors verrs = pimItem.save();
            List<ValidationErrors> errs = new ArrayList<ValidationErrors>();
            errs.add(verrs);
            response.setStatus(checkErrors(item.getPrimaryKey(), results, errs, "Create new item"));
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Create a lookup table entry with supplied attribute values
     * 
     * @param lookupKey
     *            - key of item to create
     * @param lookupName
     *            - catalog containing the item
     * @param attrNames
     *            - names of attributes to update
     * @param types
     *            - corresponding types of attributes to update
     * @param attrValues
     *            - corresponding values of attributes to set
     * @return String[] - array of validation error messages
     */
    @WebMethod
    public ServiceResponse createLookupEntry(Context ctx, Item item)
    {
        ServiceResponse response = new ServiceResponse();
        
        String lookupKey = item.getPrimaryKey();
        String lookupName = item.getContainerName();
        System.out.println("SampleService.createLookupEntry: " + lookupKey + " in Lookup Table: " + lookupName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            LookupTableManager lookupManager = pimContext.getLookupTableManager();
            LookupTable lkup = lookupManager.getLookupTable(lookupName);
            String specName = lkup.getSpec().getName();
            LookupTableEntry entry = lkup.createEntry();

            for (Attribute attribute:item.getAttributes())
            {
                if (attribute.getType() != null)
                {
                    setLookupEntryAttributeValue(pimContext, entry, specName, attribute);
                }
            }
            ValidationErrors verrs = entry.save();
            List<ValidationErrors> errs = new ArrayList<ValidationErrors>();
            errs.add(verrs);
            response.setStatus(checkErrors(lookupKey, results, errs, "Create new lookup entry"));
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Create a category with supplied attribute values
     * 
     * @param categoryPath
     *            - key of item to create
     * @param hierarchyName
     *            - catalog containing the item
     * @param attrNames
     *            - names of attributes to update
     * @param types
     *            - corresponding types of attributes to update
     * @param attrValues
     *            - corresponding values of attributes to set
     * @return String[] - array of validation error messages
     */
    @WebMethod
    public ServiceResponse createCategory(Context ctx, Category category)
    {
        //String categoryPath, String hierarchyName, String[] attrNames,
        //String[] types, String[] attrValues
        ServiceResponse response = new ServiceResponse();
        
        String categoryPath = category.getCategoryPath();
        String hierarchyName = category.getContainerName();
        System.out.println("SampleService.createCategory: " + categoryPath + " in Hierarchy: " + hierarchyName);
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            HierarchyManager hierarchyManager = pimContext.getHierarchyManager();
            Hierarchy hier = hierarchyManager.getHierarchy(hierarchyName);
            String specName = hier.getPrimarySpec().getName();
            String parentCategoryPath = "";
            if (categoryPath.indexOf("/") != -1)
            {
                parentCategoryPath = categoryPath.substring(0, categoryPath.lastIndexOf("/"));
                categoryPath = categoryPath.substring(categoryPath.lastIndexOf("/") + 1);
            }

            com.ibm.pim.hierarchy.category.Category pimCategory = hier.createCategory(parentCategoryPath, "/", categoryPath);

            for (Attribute attribute:category.getAttributes())
            {
                if (attribute.getType() != null)
                {
                    setCategoryAttributeValue(pimContext, pimCategory, specName, attribute);
                }
            }
            List<ValidationErrors> verrs = hier.save();
            response.setStatus(checkErrors(categoryPath, results, verrs, "Create new category"));
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    private int checkErrors(String entryKey, List<String> results, List<ValidationErrors> listValodationErrors, String messagePrefix)
    {
        int status = ServiceResponse.SUCCEEDED;
        if (listValodationErrors != null)
        {
            for (ValidationErrors validationErrors : listValodationErrors)
            {
            	if (validationErrors != null)
            	{
	                for (ValidationError validationError : validationErrors.getErrors())
	                {
	                    String validationErrorMessage = validationError.getMessage();
	                    if (validationErrorMessage.equals(""))
	                    {
	                        validationErrorMessage = validationError.getAttributeInstance().getPath();
	                        validationErrorMessage += ":" + validationError.getAttributeInstance().getDisplayValue();
	                        validationErrorMessage += ":" + validationError.getType().name();
	                    }
	                    System.err.println(messagePrefix + ": " + entryKey + " failed - errors: " + validationErrorMessage);
	                    results.add(validationErrorMessage);
	                }
	                status = ServiceResponse.FAILED;
            	}
            }
        }
        else
        {
            System.out.println(messagePrefix + ": " + entryKey + " succeeded");
        }
		return status;
    }

    /**
     * Delete an item from a catalog by the catalog name and the item PK.
     * 
     * @param ctx
     * @param item
     * @return
     */
    @WebMethod
    public ServiceResponse deleteItem(Context ctx, Item item)
    {
        System.out.println("SampleService.deleteItem: " + item.getPrimaryKey() + " in Catalog: " + item.getContainerName());
        
        ServiceResponse response = new ServiceResponse();
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            CatalogManager catalogManager = pimContext.getCatalogManager();
            Catalog ctg = catalogManager.getCatalog(item.getContainerName());
            com.ibm.pim.catalog.item.Item pimItem = ctg.getItemByPrimaryKey(item.getPrimaryKey());
            if (pimItem != null)
            {
	            pimItem.delete();
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Delete an entry from a Lookup Table
     * 
     * @param lookupKey
     *            - key of item to update
     * @param lookupName
     *            - catalog containing the item
     * @return errors
     */
    @WebMethod
    public ServiceResponse deleteLookupEntry(Context ctx, Item item)
    {
        ServiceResponse response = new ServiceResponse();
        //String lookupKey, String lookupName        
        System.out.println("SampleService.deleteLookupEntry: " + item.getPrimaryKey() + " in Lookup Table: " + item.getContainerName());
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            LookupTableManager lookupManager = pimContext.getLookupTableManager();
            LookupTable lkup = lookupManager.getLookupTable(item.getContainerName());
            LookupTableEntry entry = lkup.getLookupTableEntry(item.getPrimaryKey());
            if (entry != null)
            {
	            entry.delete();
            }
            response.setStatus(ServiceResponse.SUCCEEDED);
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Delete a category from a hierarchy
     * 
     * @param categoryPath
     *            - key of item to update
     * @param hierarchyName
     *            - catalog containing the item
     * @return errors
     */
    @WebMethod
    public ServiceResponse deleteCategory(Context ctx, Category category)
    {
        ServiceResponse response = new ServiceResponse();
        response.setRequest(new ServiceRequest(ctx));
        
        System.out.println("SampleService.deleteCategory: " + category.getCategoryPath() + " in Hierarchy: " + category.getContainerName());
        com.ibm.pim.context.Context pimContext = null;
        List<String> results = new ArrayList<String>();
        try
        {
            pimContext = ctx.toPIMObject();
            HierarchyManager hierarchyManager = pimContext.getHierarchyManager();
            Hierarchy hier = hierarchyManager.getHierarchy(category.getContainerName());
            com.ibm.pim.hierarchy.category.Category pimCategory = hier.getCategoryByPath(category.getCategoryPath());
            if (pimCategory != null)
            {
	            pimCategory.delete();
            }
            List<ValidationErrors> verrs = hier.save();
            response.setStatus(checkErrors(category.getCategoryPath(), results, verrs, "Create new category"));
        }
        catch (Exception e)
        {
            results.add("ERROR:" + e.getMessage());
            response.setStatus(ServiceResponse.FAILED);
        }
        finally
        {
            // Cleanup.
            if(pimContext!=null)
                pimContext.cleanUp();
        }
        response.setResults(results);
        
        return response;
    }

    /**
     * Get the value for the given Items' attribute path
     * 
     * @param item
     * @param attributePath
     * @return current attribute value
     */
    private String getItemAttributeValue(com.ibm.pim.catalog.item.Item item, String attributePath)
    {
        AttributeInstance attrInstance = item.getAttributeInstance(attributePath);

        return getAttributeValue(attrInstance);
    }

    /**
     * Get the value for the given Lookup Entry attribute path
     * 
     * @param entry
     * @param attributePath
     * @return current entry value
     */
    private String getLookupEntryAttributeValue(LookupTableEntry entry, String attributePath)
    {
        AttributeInstance attrInstance = entry.getAttributeInstance(attributePath);

        return getAttributeValue(attrInstance);
    }

    /**
     * Get value for the given Category' attribute path
     * 
     * @param cat
     *            Category
     * @param attributePath
     * @return current attribute value
     */
    private String getCategoryAttributeValue(com.ibm.pim.hierarchy.category.Category cat, String attributePath)
    {
        AttributeInstance attrInstance = cat.getAttributeInstance(attributePath);

        return getAttributeValue(attrInstance);
    }

    /**
     * Get value for the attribute instance
     * 
     * @param attrInstance
     * @return current attribute value
     */
    private String getAttributeValue(AttributeInstance attrInstance)
    {
        AttributeDefinition ad = attrInstance.getAttributeDefinition();
        String attrValueString = null;

        if (attrInstance == null)
            attrValueString = "";
        else if (ad.getType().equals(AttributeDefinition.Type.GROUPING))
            attrValueString = "GROUPING";
        else if (attrInstance.getValue() != null)
        {
            if (ad.getType().equals(AttributeDefinition.Type.CURRENCY))
            {
                String currencyCode = ad.getProperty(AttributeDefinitionProperty.Name.CURRENCY).getValue();
                attrValueString = currencyCode + attrInstance.getValue().toString();
            }
            else if (ad.getType().equals(AttributeDefinition.Type.DATE))
            {
                Date attributeDateValue = (Date) (attrInstance.getValue());
                attrValueString = String.valueOf(attributeDateValue.getTime());
            }
            else if (ad.getType().equals(AttributeDefinition.Type.RELATIONSHIP))
            {
                com.ibm.pim.catalog.item.Item relItem = (com.ibm.pim.catalog.item.Item) (attrInstance.getValue());
                attrValueString = relItem.getCatalog().getName() + ":" + relItem.getPrimaryKey();
            }
            else if (ad.getType().equals(AttributeDefinition.Type.TIMEZONE))
                attrValueString = attrInstance.getDisplayValue();
            else if (ad.getType().equals(AttributeDefinition.Type.NUMBER)
                     || ad.getType().equals(AttributeDefinition.Type.NUMBER_ENUMERATION))
            {
                int precision = Integer.valueOf(ad.getProperty(AttributeDefinitionProperty.Name.PRECISION).getValue()).intValue();
                Double d = (Double) attrInstance.getValue();
                NumberFormat nf = NumberFormat.getInstance();
                nf.setMinimumFractionDigits(precision);
                attrValueString = nf.format(d.doubleValue());
            }
            else
                attrValueString = attrInstance.getValue().toString();
        }
        else if (ad.getType().equals(AttributeDefinition.Type.TIMEZONE)
                 || ad.getType().equals(AttributeDefinition.Type.LOOKUP_TABLE)
                 || ad.getType().equals(AttributeDefinition.Type.NUMBER_ENUMERATION)
                 || ad.getType().equals(AttributeDefinition.Type.STRING_ENUMERATION)
        // || ad.getType().equals(AttributeDefinition.Type.FLAG)
        )
            attrValueString = "-NONE-";
        else
            attrValueString = "";

        return attrValueString;
    }

    /**
     * @param item
     *            - item required
     * @param attributePath
     *            - attribute required
     * @return value of attribute in the format
     *         specName/attributeName*attributeType*attributeValue
     */
    private String getDynamicItemAttributeValue(com.ibm.pim.catalog.item.Item item, String attributePath)
    {
        // System.out.print("getting item attribute value: " + attributeName);
        // CatalogManager catalogManager = context.getCatalogManager();
        // Catalog ctg = catalogManager.getCatalog(catalogName);
        // String specName = ctg.getSpec().getName();
        // Item item = ctg.getItemByPrimaryKey(itemKey);
        AttributeInstance attrInstance = item.getAttributeInstance(attributePath);
        AttributeDefinition ad = attrInstance.getAttributeDefinition();
        // System.out.println(":" + ad.getType().name());

        String attrValueString = attributePath + "*";

        if (ad.getType().equals(AttributeDefinition.Type.GROUPING))
            attrValueString += "GROUPING*";
        else if (attrInstance.getValue() != null)
        {
            if (ad.getType().equals(AttributeDefinition.Type.CURRENCY))
            {
                String currencyCode = ad.getProperty(AttributeDefinitionProperty.Name.CURRENCY).getValue();
                attrValueString += "CURRENCY*" + currencyCode + attrInstance.getValue().toString();
            }
            else if (ad.getType().equals(AttributeDefinition.Type.DATE))
            {
                Date attributeDateValue = (Date) (attrInstance.getValue());
                attrValueString += "DATE*" + String.valueOf(attributeDateValue.getTime());
            }
            else if (ad.getType().equals(AttributeDefinition.Type.RELATIONSHIP))
            {
                com.ibm.pim.catalog.item.Item relItem = (com.ibm.pim.catalog.item.Item) (attrInstance.getValue());
                attrValueString += "RELATIONSHIP*" + relItem.getCatalog().getName() + ":" + relItem.getPrimaryKey();
            }
            else if (ad.getType().equals(AttributeDefinition.Type.TIMEZONE))
                attrValueString += "TIMEZONE*" + attrInstance.getDisplayValue();
            else if (ad.getType().equals(AttributeDefinition.Type.NUMBER)
                     || ad.getType().equals(AttributeDefinition.Type.NUMBER_ENUMERATION))
            {
                int precision = Integer.valueOf(ad.getProperty(AttributeDefinitionProperty.Name.PRECISION).getValue()).intValue();
                Double d = (Double) attrInstance.getValue();
                NumberFormat nf = NumberFormat.getInstance();
                nf.setMinimumFractionDigits(precision);
                attrValueString += ad.getType().name() + "*" + nf.format(d.doubleValue());
            }
            else
                attrValueString += ad.getType().name() + "*" + attrInstance.getValue().toString();
        }
        else // attrValue == null
        if (ad.getType().equals(AttributeDefinition.Type.FLAG) || ad.getType().equals(AttributeDefinition.Type.LOOKUP_TABLE)
            || ad.getType().equals(AttributeDefinition.Type.NUMBER_ENUMERATION)
            || ad.getType().equals(AttributeDefinition.Type.STRING_ENUMERATION))
            attrValueString += ad.getType().name() + "*" + "-NONE-";
        else
            attrValueString += ad.getType().name() + "*";
        return attrValueString;
    }

    /**
     * Set the given Items' attribute values
     * 
     * @param pimItem
     * @param attributeName
     *            full attribute path
     * @param attributeType
     * @param attributeValue
     */
    private void setItemAttributeValue(com.ibm.pim.context.Context pimContext, com.ibm.pim.catalog.item.Item pimItem, Attribute attribute)
    {
        String attributeName = attribute.getName();
        String attributeType = attribute.getType();
        String attributeValue = attribute.getValue();
        if (attributeValue == null || attributeValue.equals("-NONE-"))
        {
            attributeValue = "";
        }
        
        System.out.println("Set:" + attributeName + ":" + attributeType +":" + attributeValue);
        AttributeInstance pimAttributeInstance = pimItem.getAttributeInstance(attributeName);
        if (pimAttributeInstance != null)
        {
            setValueForAttributeInstance(pimContext, pimAttributeInstance, attributeType, attributeValue);
        }
        else
        {
            pimItem.setAttributeValue(attributeName, attributeValue);
        }
    }

    /**
     * Set the value according the the type of attribute
     * 
     * @param attributeInstance
     *            instance to be set
     * @param attributeType
     * @param attributeValue
     */
    private void setValueForAttributeInstance(com.ibm.pim.context.Context pimContext, AttributeInstance attributeInstance, String attributeType, String attributeValue)
    {
        // if (attributeValue != null)
        if (!attributeValue.equalsIgnoreCase("-NONE-") && !attributeValue.equalsIgnoreCase(""))
        {
            if (attributeType.equalsIgnoreCase("CURRENCY"))
            {
                attributeInstance.setValue(attributeValue);
            }
            else if (attributeType.equalsIgnoreCase("DATE"))
            {
                attributeInstance.setValue(new Date(new Long(attributeValue).longValue()));
            }
            else if (attributeType.equalsIgnoreCase("RELATIONSHIP"))
            {
                String catalogName = null;
                String relItemPk = null;
                if (attributeValue.indexOf(':') != -1)
                {
                    // catalogName:itemPk specified
                    catalogName = attributeValue.substring(0, attributeValue.indexOf(':'));
                    relItemPk = attributeValue.substring(attributeValue.indexOf(':') + 1);
                    CatalogManager catalogManager = pimContext.getCatalogManager();
                    Catalog ctg = catalogManager.getCatalog(catalogName);
                    com.ibm.pim.catalog.item.Item relItem = ctg.getItemByPrimaryKey(relItemPk);
                    attributeInstance.setValue(relItem);
                }
            }
            else if (attributeType.equalsIgnoreCase("TIMEZONE"))
            {
                TimezoneResolver tzr = pimContext.getTimezoneResolver();
                attributeInstance.setValue(tzr.getEncodedFormFromTimezoneString(attributeValue));
            }
            else if (attributeType.equalsIgnoreCase("PASSWORD") && !attributeValue.startsWith("*"))
                // only if user has updated it
                attributeInstance.setValue(attributeValue);
            else if (attributeType.equalsIgnoreCase("SEQUENCE"))
                ; // do nothing - cannot update sequence fields
            else
                attributeInstance.setValue(attributeValue);
        }
        else // attrValue = -NONE- or empty string
        if (attributeType.equalsIgnoreCase("RELATIONSHIP"))
            attributeInstance.setValue(null);
        else
            attributeInstance.setValue("");
    }

    /**
     * Set the given Lookup Entry' attribute value
     * 
     * @param entry
     * @param specName
     * @param attributeName
     * @param attributeType
     * @param attributeValue
     */
    private void setLookupEntryAttributeValue(com.ibm.pim.context.Context pimContext, LookupTableEntry entry, String specName, Attribute attribute)
    {
        String attributeValue = attribute.getValue();
        if (attributeValue == null || attributeValue.equals("-NONE-"))
            attributeValue = "";
        
        String attributeName = attribute.getName();
        AttributeInstance attrInstance = entry.getAttributeInstance(specName + "/" + attributeName);
        if (attrInstance != null)
            setValueForAttributeInstance(pimContext, attrInstance, attribute.getType(), attributeValue);
        else
            entry.setAttributeValue(specName + "/" + attributeName, attributeValue);
    }

    /*
     * Set the given Category' attribute value
     */
    private void setCategoryAttributeValue(com.ibm.pim.context.Context pimContext, com.ibm.pim.hierarchy.category.Category cat, String specName, Attribute attribute)
    {
        String attributeValue = attribute.getValue();
        if (attributeValue == null || attributeValue.equals("-NONE-"))
        {
            attributeValue = "";
        }
        
        String attributeName = attribute.getName();
        AttributeInstance attrInstance = cat.getAttributeInstance(specName + "/" + attributeName);
        if (attrInstance != null)
        {
            setValueForAttributeInstance(pimContext, attrInstance, attribute.getType(), attributeValue);
        }
        else
        {
            cat.setAttributeValue(specName + "/" + attributeName, attributeValue);
        }
    }

}
